# -*- coding: utf-8 -*- #
# Copyright 2016 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Common flags for some of the DNS commands."""


from googlecloudsdk.calliope import actions
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.calliope.concepts import concepts
from googlecloudsdk.command_lib.util import completers
from googlecloudsdk.command_lib.util.apis import arg_utils
from googlecloudsdk.command_lib.util.concepts import concept_parsers


class BetaKeyCompleter(completers.ListCommandCompleter):

  def __init__(self, **kwargs):
    super(BetaKeyCompleter, self).__init__(
        collection='dns.dnsKeys',
        api_version='v1beta2',
        list_command='beta dns dns-keys list --format=value(keyTag)',
        parse_output=True,
        flags=['zone'],
        **kwargs,
    )


class KeyCompleter(completers.ListCommandCompleter):

  def __init__(self, **kwargs):
    super(KeyCompleter, self).__init__(
        collection='dns.dnsKeys',
        api_version='v1',
        list_command='dns dns-keys list --format=value(keyTag)',
        parse_output=True,
        flags=['zone'],
        **kwargs,
    )


class ManagedZoneCompleter(completers.ListCommandCompleter):

  def __init__(self, **kwargs):
    super(ManagedZoneCompleter, self).__init__(
        collection='dns.managedZones',
        list_command='dns managed-zones list --uri',
        **kwargs,
    )


def GetKeyArg(help_text='The DNS key identifier.', is_beta=False):
  return base.Argument(
      'key_id',
      metavar='KEY-ID',
      completer=BetaKeyCompleter if is_beta else KeyCompleter,
      help=help_text,
  )


def GetSkipSoaUpdateArg():
  return base.Argument(
      '--skip-soa-update',
      action='store_true',
      default=False,
      help=(
          'Skip incrementing the serial number of the SOA record when making'
          ' changes to the record-sets.'
      ),
  )


def GetDnsZoneArg(help_text):
  return base.Argument(
      'dns_zone',
      metavar='ZONE_NAME',
      completer=ManagedZoneCompleter,
      help=help_text,
  )


def ZoneAttributeConfig():
  return concepts.ResourceParameterAttributeConfig(
      name='zone', help_text='The Cloud DNS zone for the {resource}.'
  )


def GetZoneResourceSpec():
  return concepts.ResourceSpec(
      'dns.managedZones',
      resource_name='zone',
      managedZone=ZoneAttributeConfig(),
      project=concepts.DEFAULT_PROJECT_ATTRIBUTE_CONFIG,
      disable_auto_completers=False,
  )


def GetZoneResourceArg(help_text, positional=True, plural=False):
  arg_name = 'zones' if plural else 'zone'
  return concept_parsers.ConceptParser.ForResource(
      arg_name if positional else '--{}'.format(arg_name),
      GetZoneResourceSpec(),
      help_text,
      plural=plural,
      required=True,
  )


def GetZoneArg(
    help_text='Name of the managed zone whose record sets you want to manage.',
    hide_short_zone_flag=False,
):
  """Returns the managed zone arg."""
  if hide_short_zone_flag:
    zone_group = base.ArgumentGroup(required=True)
    zone_group.AddArgument(
        base.Argument('--zone', completer=ManagedZoneCompleter, help=help_text)
    )
    zone_group.AddArgument(
        base.Argument(
            '-z',
            dest='zone',
            completer=ManagedZoneCompleter,
            help=help_text,
            hidden=True,
        )
    )
    return zone_group
  else:
    return base.Argument(
        '--zone',
        '-z',
        completer=ManagedZoneCompleter,
        help=help_text,
        required=True,
    )


def GetManagedZonesDnsNameArg():
  return base.Argument(
      '--dns-name',
      required=True,
      help='The DNS name suffix that will be managed with the created zone.',
  )


def GetZoneIdArg(
    help_text=(
        'The unique system generated id for the peering zone to deactivate.'
    ),
):
  return base.Argument('--zone-id', required=True, help=help_text)


def GetPeeringZoneListArg():
  return base.Argument(
      '--target-network',
      required=True,
      help=(
          'The network url of the Google Compute Engine private network '
          'to forward queries to.'
      ),
  )


def GetManagedZonesDescriptionArg():
  return base.Argument(
      '--description',
      required=False,
      help='Short description for the managed zone.',
  )


def GetDnsSecStateFlagMapper(messages, api_version='v1'):
  """Returns a ChoiceEnumMapper for the --dnssec-state flag.

  The v2 DNS API uses uppercase enum values for dnssec-state (e.g. 'ON'),
  while v1 uses lowercase. To maintain a consistent user experience, gcloud
  accepts lowercase values for all API versions. The `ChoiceEnumMapper`
  validates that custom mapping keys are valid API enum values. For v2, this
  requires mapping the uppercase API enum to the lowercase, user-facing
  choice. The mapper then handles translating the user's input to the
  correct value for the API request.

  Args:
    messages: The messages module for the API version.
    api_version: The API version to use (e.g., 'v1', 'v2').

  Returns:
    A ChoiceEnumMapper for the --dnssec-state flag.
  """
  v1_mappings = {
      'off': ('off', 'Disable DNSSEC for the managed zone.'),
      'on': ('on', 'Enable DNSSEC for the managed zone.'),
      'transfer': (
          'transfer',
          'Enable DNSSEC and allow transferring a signed zone in or out.',
      ),
  }
  if api_version == 'v2':
    custom_mappings = {k.upper(): v for k, v in v1_mappings.items()}
  else:
    custom_mappings = v1_mappings
  return arg_utils.ChoiceEnumMapper(
      '--dnssec-state',
      messages.ManagedZoneDnsSecConfig.StateValueValuesEnum,
      custom_mappings=custom_mappings,
      help_str='The DNSSEC state for this managed zone.',
  )


def GetDoeFlagMapper(messages):
  return arg_utils.ChoiceEnumMapper(
      '--denial-of-existence',
      messages.ManagedZoneDnsSecConfig.NonExistenceValueValuesEnum,
      help_str='Requires DNSSEC enabled.',
  )


def GetKeyAlgorithmFlagMapper(key_type, messages):
  return arg_utils.ChoiceEnumMapper(
      '--{}-algorithm'.format(key_type),
      messages.DnsKeySpec.AlgorithmValueValuesEnum,
      help_str=(
          'String mnemonic specifying the DNSSEC algorithm of the '
          'key-signing key. Requires DNSSEC enabled'
      ),
  )


def AddCommonManagedZonesDnssecArgs(parser, messages, api_version='v1'):
  """Add Common DNSSEC flags for the managed-zones group."""
  GetDnsSecStateFlagMapper(messages, api_version).choice_arg.AddToParser(parser)
  GetDoeFlagMapper(messages).choice_arg.AddToParser(parser)
  GetKeyAlgorithmFlagMapper('ksk', messages).choice_arg.AddToParser(parser)
  GetKeyAlgorithmFlagMapper('zsk', messages).choice_arg.AddToParser(parser)
  parser.add_argument(
      '--ksk-key-length',
      type=int,
      help='Length of the key-signing key in bits. Requires DNSSEC enabled.',
  )
  parser.add_argument(
      '--zsk-key-length',
      type=int,
      help='Length of the zone-signing key in bits. Requires DNSSEC enabled.',
  )


def GetManagedZoneVisibilityArg():
  return base.Argument(
      '--visibility',
      choices=['public', 'private'],
      default='public',
      help=(
          'Visibility of the zone. Public zones are visible to the public '
          'internet. Private zones are only visible in your internal '
          'networks denoted by the `--networks` flag.'
      ),
  )


def GetManagedZoneNetworksArg():
  return base.Argument(
      '--networks',
      metavar='NETWORK',
      type=arg_parsers.ArgList(),
      help=(
          'List of networks that the zone should be visible in if the zone '
          'visibility is [private].'
      ),
  )


def GetManagedZoneGkeClustersArg():
  return base.Argument(
      '--gkeclusters',
      metavar='GKECLUSTERS',
      type=arg_parsers.ArgList(),
      help=(
          'List of GKE clusters that the zone should be visible in if the zone '
          'visibility is [private].'
      ),
  )


def GetDnsPeeringArgs():
  """Return arg group for DNS Peering flags."""
  peering_group = base.ArgumentGroup(required=False)
  target_network_help_text = (
      'Network ID of the Google Compute Engine private network to forward'
      ' queries to.'
  )
  target_project_help_text = (
      'Project ID of the Google Compute Engine private network to forward'
      ' queries to.'
  )
  peering_group.AddArgument(
      base.Argument(
          '--target-network', required=True, help=target_network_help_text
      )
  )
  peering_group.AddArgument(
      base.Argument(
          '--target-project', required=True, help=target_project_help_text
      )
  )
  return peering_group


def GetForwardingTargetsArg():
  return base.Argument(
      '--forwarding-targets',
      type=arg_parsers.ArgList(),
      metavar='IP_ADDRESSES',
      help=(
          'List of IPv4/IPv6 addresses or one domain name of the target name'
          ' server that the zone will forward queries to. Ignored for `public`'
          ' visibility. Non-RFC1918 addresses will forward to the target'
          ' through the Internet. RFC1918 addresses will forward through the'
          ' VPC.'
      ),
  )


def GetPrivateForwardingTargetsArg():
  return base.Argument(
      '--private-forwarding-targets',
      type=arg_parsers.ArgList(),
      metavar='IP_ADDRESSES',
      help=(
          'List of IPv4/IPv6 addresses or one domain name of the target name'
          ' server that the zone will forward queries to. Ignored for `public`'
          ' visibility. All addresses specified for this parameter will be'
          ' reached through the VPC.'
      ),
  )


def GetReverseLookupArg():
  return base.Argument(
      '--managed-reverse-lookup',
      action='store_true',
      default=None,
      help=(
          'Specifies whether this zone is a managed reverse lookup zone, '
          'required for Cloud DNS to correctly resolve Non-RFC1918 PTR records.'
      ),
  )


def GetServiceDirectoryArg():
  return base.Argument(
      '--service-directory-namespace',
      required=False,
      help=(
          'The fully qualified URL of the service directory namespace that '
          'should be associated with the zone. Ignored for `public` visibility '
          'zones.'
      ),
  )


# Policy Flags
def GetPolicyDescriptionArg(required=False):
  return base.Argument(
      '--description', required=required, help='A description of the policy.'
  )


def GetPolicyNetworksArg(required=False):
  return base.Argument(
      '--networks',
      type=arg_parsers.ArgList(),
      metavar='NETWORKS',
      required=required,
      help=(
          'The comma separated list of network names to associate with '
          'the policy.'
      ),
  )


def GetPolicyInboundForwardingArg():
  return base.Argument(
      '--enable-inbound-forwarding',
      action='store_true',
      help=(
          'Specifies whether to allow networks bound to this policy to '
          'receive DNS queries sent by VMs or applications over VPN '
          'connections. Defaults to False.'
      ),
  )


def GetPolicyLoggingArg():
  return base.Argument(
      '--enable-logging',
      action='store_true',
      help='Specifies whether to enable query logging. Defaults to False.',
  )


def GetPolicyAltNameServersArg():
  return base.Argument(
      '--alternative-name-servers',
      type=arg_parsers.ArgList(),
      metavar='NAME_SERVERS',
      help=(
          'List of alternative name servers to forward to. Non-RFC1918 '
          'addresses will forward to the target through the Internet.'
          'RFC1918 addresses will forward through the VPC.'
      ),
  )


def GetPolicyPrivateAltNameServersArg():
  return base.Argument(
      '--private-alternative-name-servers',
      type=arg_parsers.ArgList(),
      metavar='NAME_SERVERS',
      help=(
          'List of alternative name servers to forward to. All addresses'
          ' specified for this parameter will be reached through the VPC.'
      ),
  )


def GetEnableDns64AllQueriesArg(update=False):
  """Returns the enable dns64 all queries arg for create or update."""
  if not update:
    return base.Argument(
        '--enable-dns64-all-queries',
        action='store_true',
        default=None,
        help=(
            'Specifies whether to allow networks bound to this policy to use '
            'DNS64 for IPv6-only VM instances.'
        ),
    )
  else:
    return base.Argument(
        '--enable-dns64-all-queries',
        action=arg_parsers.StoreTrueFalseAction,
        help=(
            'Specifies whether to allow networks bound to this policy to use '
            'DNS64 for IPv6-only VM instances.'
        ),
    )


## ResourceRecordSets flags.
def GetResourceRecordSetsNameArg():
  return base.Argument(
      'name', metavar='DNS_NAME', help='DNS or domain name of the record-set.'
  )


def GetResourceRecordSetsTypeArg(required=False):
  return base.Argument(
      '--type',
      required=required,
      help='DNS record type of the record-set (e.g. A, AAAA, MX etc.).',
  )


def GetResourceRecordSetsTtlArg(required=False):
  return base.Argument(
      '--ttl',
      type=int,
      required=required,
      help='TTL (time to live) for the record-set.',
  )


def GetResourceRecordSetsRrdatasArg(required=False):
  return base.Argument(
      '--rrdatas',
      metavar='RRDATA',
      required=required,
      type=arg_parsers.ArgList(),
      help=(
          'DNS data (Address/CNAME/MX info, etc.) of the record-set. '
          'This is RDATA; the format of this information varies depending '
          'on the type and class of the resource record.'
      ),
  )


def GetResourceRecordSetsRrdatasArgGroup(use_deprecated_names=False):
  """Returns arg group for rrdatas flags.

  Args:
    use_deprecated_names: If true, uses snake_case names for flags
      --routing-policy-type and --routing-policy-data, --routing_policy_type and
      --routing_policy_data.  This group is defined with required=True and
      mutex=True, meaning that exactly one of these two arg configurations must
      be specified: --rrdatas --routing-policy-type AND --routing-policy-data
  """
  # Group containing the primary backup config.
  routing_policy_backup_data_group = base.ArgumentGroup(
      mutex=True,
      required=True,
      help=(
          'Routing policy backup data arguments for the primary backup routing'
          ' policy. Specify either --routing-policy-backup-data or'
          ' --routing-policy-backup-item, but not both.'
      ),
  )
  routing_policy_backup_data_group.AddArgument(
      GetResourceRecordSetsRoutingPolicyBackupDataArg()
  )
  routing_policy_backup_data_group.AddArgument(GetRoutingPolicyBackupItemArg())

  primary_backup_data_group = base.ArgumentGroup(
      help='Configuration for primary backup routing policy'
  )
  primary_backup_data_group.AddArgument(
      GetResourceRecordSetsRoutingPolicyPrimaryDataArg(required=True)
  )
  primary_backup_data_group.AddArgument(routing_policy_backup_data_group)
  primary_backup_data_group.AddArgument(
      GetResourceRecordSetsRoutingPolicyBackupDataTypeArg(required=True)
  )
  primary_backup_data_group.AddArgument(
      GetResourceRecordSetsBackupDataTrickleRatio(required=False)
  )

  # This group dictates that we should either have a primary backup config,
  # or, a wrr or geo config
  policy_data_group = base.ArgumentGroup(
      required=True,
      mutex=True,
      help=(
          'Routing policy data arguments. Allows setting one of'
          ' [routing-policy-data, routing-policy-item,'
          ' [routing-policy-primary-data, [routing-policy-backup-data,'
          ' routing-policy-backup-item]]]'
      ),
  )
  policy_data_group.AddArgument(
      GetResourceRecordSetsRoutingPolicyDataArg(
          deprecated_name=use_deprecated_names
      )
  )
  policy_data_group.AddArgument(primary_backup_data_group)
  policy_data_group.AddArgument(GetRoutingPolicyItemArg())
  # Declare optional routing policy group. If group specified, must contain
  # both routing-policy-type and routing-policy-data args.
  policy_group = base.ArgumentGroup(
      required=False,
      help=(
          'Routing policy arguments. --routing-policy-type should be specified'
          ' exactly when one of --routing-policy-data, --routing-policy-item,'
          ' or --routing-policy-primary-data is set.'
      ),
  )
  health_checking_group = base.ArgumentGroup(
      required=False,
      mutex=True,
      help=(
          'Health checking arguments. You can specify one of --health-check or'
          ' --enable-health-checking, but not both.'
      ),
  )
  health_checking_group.AddArgument(
      GetResourceRecordSetsEnableHealthChecking(required=False)
  )
  health_checking_group.AddArgument(GetHealthCheckArg(required=False))

  policy_group.AddArgument(
      GetResourceRecordSetsRoutingPolicyTypeArg(
          required=True, deprecated_name=use_deprecated_names
      )
  )
  policy_group.AddArgument(
      GetResourceRecordSetsEnableFencingArg(required=False)
  )
  policy_group.AddArgument(health_checking_group)
  policy_group.AddArgument(policy_data_group)

  rrdatas_group = base.ArgumentGroup(
      required=True,
      mutex=True,
      help=(
          'Resource record sets arguments. Can specify either --rrdatas or both'
          ' --routing-policy-data and --routing-policy-type.'
      ),
  )
  rrdatas_group.AddArgument(GetResourceRecordSetsRrdatasArg(required=False))
  rrdatas_group.AddArgument(policy_group)

  return rrdatas_group


def GetResourceRecordSetsRoutingPolicyTypeArg(
    required=False, deprecated_name=False
):
  """Returns --routing-policy-type command line arg value."""
  flag_name = (
      '--routing_policy_type' if deprecated_name else '--routing-policy-type'
  )
  return base.Argument(
      flag_name,
      metavar='ROUTING_POLICY_TYPE',
      required=required,
      choices=['GEO', 'WRR', 'FAILOVER'],
      help=(
          'Indicates what type of routing policy is being specified. As of this'
          ' time, this field can take on either "WRR" for weighted round robin,'
          ' "GEO" for geo location, or "FAILOVER" for a primary-backup'
          ' configuration. This field cannot be modified - once a policy has a'
          ' chosen type, the only way to change it is to delete the policy and'
          ' add a new one with the different type.'
      ),
  )


def GetResourceRecordSetsEnableFencingArg(required=False):
  """Returns --enable-geo-fencing command line arg value."""
  return base.Argument(
      '--enable-geo-fencing',
      action='store_true',
      required=required,
      help='Specifies whether to enable fencing for geo queries.',
  )


def GetResourceRecordSetsBackupDataTrickleRatio(required=False):
  """Returns --backup-data-trickle-ratio command line arg value."""
  return base.Argument(
      '--backup-data-trickle-ratio',
      type=float,
      required=required,
      help=(
          'Specifies the percentage of traffic to send to the backup targets'
          ' even when the primary targets are healthy.'
      ),
  )


def GetResourceRecordSetsRoutingPolicyBackupDataTypeArg(required=True):
  """Returns --routing_policy_backup_data_type command line arg value."""
  return base.Argument(
      '--routing-policy-backup-data-type',
      metavar='ROUTING_POLICY_BACKUP_DATA_TYPE',
      required=required,
      choices=['GEO'],
      help=(
          'For FAILOVER routing policies, the type of routing policy the backup'
          ' data uses. Currently, this must be GEO'
      ),
  )


def GetResourceRecordSetsEnableHealthChecking(required=False):
  """Returns --enable-health-checking command line arg value."""
  return base.Argument(
      '--enable-health-checking',
      action='store_true',
      required=required,
      help='Required if specifying forwarding rule names for rrdata.',
  )


def GetResourceRecordSetsRoutingPolicyPrimaryDataArg(required=False):
  """Returns --routing-policy-primary-data command line arg value."""

  def RoutingPolicyPrimaryDataArg(routing_policy_primary_data):
    """Converts --routing-policy-primary-data flag value to a list of policy data items.

    Args:
      routing_policy_primary_data: String value specified in the
        --routing-policy-primary-data flag.

    Returns:
      A list of forwarding configs in the following format:
        ['config1@region1', 'config2@region2', 'config3' ]
      OR a list of IP addresses in the following format:
        ['1.1.1.1', '2.2.2.2', '3.3.3.3']
    """

    # Grab each policy data item by splitting on ','
    return routing_policy_primary_data.split(',')

  return base.Argument(
      '--routing-policy-primary-data',
      metavar='ROUTING_POLICY_PRIMARY_DATA',
      required=required,
      type=RoutingPolicyPrimaryDataArg,
      help=(
          'The primary configuration for a primary backup routing policy. '
          'This configuration is a list of forwarding rules of the format '
          '"FORWARDING_RULE_NAME", "FORWARDING_RULE_NAME@scope", or the full '
          'resource path of the forwarding rule.'
      ),
  )


def GetResourceRecordSetsRoutingPolicyBackupDataArg(required=False):
  """Returns --routing-policy-backup-data command line arg value."""

  def RoutingPolicyBackupDataArg(routing_policy_backup_data):
    """Converts --routing-policy-backup-data flag value to a list of policy data items.

    Args:
      routing_policy_backup_data: String value specified in the
        --routing-policy-backup-data flag.

    Returns:
      A list of policy data items in the format below:

    [
        {
          'key': <location1>,
          'values': <list of forwarding configs and rrdatas>,
        },
        {
          'key': <location2>,
          'values': <list of forwarding configs and rrdatas>,
        },
        ...
    ]
    """

    backup_data = []

    # Grab each policy data item, split by ';'
    policy_items = routing_policy_backup_data.split(';')
    for policy_item in policy_items:
      # Grab key and value from policy_item, split by ':'
      key_value_split = policy_item.split('=')

      # Ensure that there is only one key and value from the string split on ':'
      if len(key_value_split) != 2:
        raise arg_parsers.ArgumentTypeError(
            'Must specify exactly one "=" inside each policy data item'
        )
      key = key_value_split[0]
      value = key_value_split[1]

      backup_data.append({'key': key, 'values': value})

    return backup_data

  return base.Argument(
      '--routing-policy-backup-data',
      action=actions.DeprecationAction(
          '--routing-policy-backup-data',
          warn=(
              'The `--routing-policy-backup-data` flag is deprecated. Use'
              ' --routing-policy-backup-item instead.'
          ),
      ),
      metavar='ROUTING_POLICY_BACKUP_DATA',
      required=required,
      type=RoutingPolicyBackupDataArg,
      help=(
          'Specify the backup configuration for a primary backup routing'
          ' policy. This backup configuration uses the same format as the'
          ' routing-policy-data argument because it functions as another'
          ' geolocation routing policy.'
      ),
  )


def GetResourceRecordSetsRoutingPolicyDataArg(
    required=False, deprecated_name=False
):
  """Returns --routing-policy-data command line arg value."""

  def RoutingPolicyDataArgType(routing_policy_data_value):
    """Converts --routing-policy-data flag value to a list of policy data items.

    Args:
      routing_policy_data_value: String value specified in the
        --routing-policy-data flag.

    Returns:
      A list of policy data items in the format below:

    [
        {
          'key': <routing_policy_data_key1>,
          'values': <list of configs and rrdatas>,
        },
        {
          'key': <routing_policy_data_key2>,
          'values': <list of configs and rrdatas>,
        },
        ...
    ]

    Where <routing_policy_data_key> is either a weight or location name,
    depending on whether the user specified --routing-policy-type == WRR or
    --routing-policy-type == GEO, respectively. We keep
    <routing_policy_data_key> a string value, even in the case of weights
    (which will eventually be interpereted as floats). This is to keep this
    flag type generic between WRR and GEO types.
    """
    routing_policy_data = []

    # Grab each policy data item, split by ';'
    policy_items = routing_policy_data_value.split(';')
    for policy_item in policy_items:
      # Grab key and value from policy_item, split by ':'
      key_value_split = policy_item.split('=')

      # Ensure that there is only one key and value from the string split on ':'
      if len(key_value_split) != 2:
        raise arg_parsers.ArgumentTypeError(
            'Must specify exactly one "=" inside each policy data item'
        )
      key = key_value_split[0]
      value = key_value_split[1]

      routing_policy_data.append({
          'key': key,
          'values': value,
      })
    return routing_policy_data

  flag_name = (
      '--routing_policy_data' if deprecated_name else '--routing-policy-data'
  )
  return base.Argument(
      flag_name,
      action=actions.DeprecationAction(
          flag_name,
          warn=(
              f'The `{flag_name}` flag is deprecated. Use'
              ' --routing-policy-item instead.'
          ),
      ),
      metavar='ROUTING_POLICY_DATA',
      required=required,
      type=RoutingPolicyDataArgType,
      help=(
          'The routing policy data supports one of two formats below, depending'
          ' on the choice of routing-policy-type.\n\nFor --routing-policy-type'
          ' = "WRR" this flag indicates the weighted round robin policy data.'
          ' The field accepts a semicolon-delimited list of the format'
          ' "${weight_percent}=${rrdata},${rrdata}". Specify weight as a'
          ' non-negative number (0 is allowed). Ratio of traffic routed to the'
          ' target is calculated from the ratio of individual weight over the'
          ' total across all weights.\n\nFor --routing-policy-type = "GEO" this'
          ' flag indicates the geo-locations policy data. The field accepts a'
          ' semicolon-delimited list of the format'
          ' "${region}=${rrdata},${rrdata}". Each rrdata can either be an IP'
          ' address or a reference to a forwarding rule of the format'
          ' "FORWARDING_RULE_NAME", "FORWARDING_RULE_NAME@{region}",'
          ' "FORWARDING_RULE_NAME@global", or the full resource path of the'
          ' forwarding rule.'
      ),
  )


# Response Policy Flags
def GetResponsePolicyDescriptionArg(required=False):
  return base.Argument(
      '--description',
      required=required,
      help='A description of the response policy.',
  )


def GetResponsePolicyNetworksArg(required=False):
  return base.Argument(
      '--networks',
      type=arg_parsers.ArgList(),
      required=required,
      metavar='NETWORKS',
      help=(
          'The comma-separated list of network names to associate with '
          'the response policy.'
      ),
  )


CHANGES_FORMAT = 'table(id, startTime, status)'


def _FormatHealthCheckTarget(health_check_target):
  fields = (
      'ipAddress',
      'port',
      'ipProtocol',
      'networkUrl',
      'project',
      'region',
      'loadBalancerType',
  )
  return ', '.join(
      health_check_target[f] for f in fields if f in health_check_target
  )


def _FormatRrdata(routing_policy_item):
  rrdata = []
  if 'rrdatas' in routing_policy_item:
    rrdata = rrdata + routing_policy_item['rrdatas']
  if 'healthCheckedTargets' in routing_policy_item:
    rrdata = (
        rrdata
        + [
            '"{}"'.format(_FormatHealthCheckTarget(target))
            for target in routing_policy_item['healthCheckedTargets'].get(
                'internalLoadBalancers', []
            )
        ]
        + routing_policy_item['healthCheckedTargets'].get(
            'externalEndpoints', []
        )
    )
  return ','.join(rrdata)


def _FormatResourceRecordSet(rrdatas_or_routing_policy):
  """Format rrset based on rrdatas or routing policy type."""
  if 'wrr' in rrdatas_or_routing_policy:
    items = []
    for item in rrdatas_or_routing_policy['wrr']['items']:
      items.append('{}: {}'.format(item['weight'], _FormatRrdata(item)))
    data = '; '.join(items)
  elif 'geo' in rrdatas_or_routing_policy:
    items = []
    for item in rrdatas_or_routing_policy['geo']['items']:
      items.append('{}: {}'.format(item['location'], _FormatRrdata(item)))
    data = '; '.join(items)
  elif 'primaryBackup' in rrdatas_or_routing_policy:
    items = []
    for item in rrdatas_or_routing_policy['primaryBackup']['backupGeoTargets'][
        'items'
    ]:
      items.append('{}: {}'.format(item['location'], _FormatRrdata(item)))
    backup = ';'.join(items)
    primary = ','.join(
        list(
            '"{}"'.format(_FormatHealthCheckTarget(target))
            for target in rrdatas_or_routing_policy['primaryBackup'][
                'primaryTargets'
            ].get('internalLoadBalancers', [])
        )
        + rrdatas_or_routing_policy['primaryBackup']['primaryTargets'].get(
            'externalEndpoints', []
        )
    )
    data = 'Primary: {} Backup: {}'.format(primary, backup)
  else:
    data = ','.join(rrdatas_or_routing_policy)
  if 'healthCheck' in rrdatas_or_routing_policy:
    data = data + ' Health Check: {}'.format(
        rrdatas_or_routing_policy['healthCheck']
    )
  return data


RESOURCERECORDSETS_TRANSFORMS = {
    'formatrrset': _FormatResourceRecordSet,
}
RESOURCERECORDSETS_FORMAT = """
    table(
        name,
        type,
        ttl,
        firstof(rrdatas,routingPolicy).formatrrset():label=DATA)
    """


def GetResponsePolicyGkeClustersArg(required=False):
  return base.Argument(
      '--gkeclusters',
      type=arg_parsers.ArgList(),
      required=required,
      metavar='GKECLUSTERS',
      help=(
          'The comma-separated list of GKE cluster names to associate with '
          'the response policy.'
      ),
  )


# Response Policy Rule Flags
def GetResponsePolicyRulesBehaviorFlagMapper(messages):
  return arg_utils.ChoiceEnumMapper(
      '--behavior',
      messages.ResponsePolicyRule.BehaviorValueValuesEnum,
      include_filter=lambda x: x != 'behaviorUnspecified',
      help_str='The response policy rule query behavior.',
  )


def GetResponsePolicyRulesBehavior():
  return base.Argument(
      '--behavior',
      choices=['behaviorUnspecified', 'bypassResponsePolicy'],
      help='The response policy rule query behavior.',
  )


def AddResponsePolicyRulesBehaviorFlagArgs(parser, messages):
  GetResponsePolicyRulesBehaviorFlagMapper(messages).choice_arg.AddToParser(
      parser
  )


def GetLocalDataResourceRecordSets():
  return base.Argument(
      '--local-data',
      type=arg_parsers.ArgDict(
          spec={'name': str, 'type': str, 'ttl': int, 'rrdatas': str}
      ),
      metavar='LOCAL_DATA',
      action='append',
      help="""\
    All resource record sets for this selector, one per resource record
    type. The name must match the dns_name.

    This is a repeated argument that can be specified multiple times to specify
    multiple local data rrsets.
    (e.g. --local-data=name="zone.com.",type="A",ttl=21600,rrdata="1.2.3.4 "
    --local-data=name="www.zone.com.",type="CNAME",ttl=21600,rrdata="1.2.3.4|5.6.7.8")

    *name*::: The DnsName of a resource record set.

    *type*::: Type of all resource records in this set. For example, A, AAAA, SOA, MX,
    NS, TXT ...

    *ttl*::: Number of seconds that this ResourceRecordSet can be cached by resolvers.

    *rrdatas*::: The list of datas for this record, split by "|".
    """,
  )


def GetResponsePolicyRuleBehavior():
  return base.Argument('--behavior', type=enumerate)


def GetManagedZoneLoggingArg():
  return base.Argument(
      '--log-dns-queries',
      action=arg_parsers.StoreTrueFalseAction,
      help='Specifies whether to enable query logging. Defaults to False.',
  )


def GetResponsePolicyNameArg(positional=True, required=True):
  if positional:
    return base.Argument(
        'response_policy',
        type=str,
        metavar='RESPONSE_POLICY_NAME',
        help='Name of the response policy.',
    )
  else:
    return base.Argument(
        '--response_policy',
        type=str,
        required=required,
        metavar='RESPONSE_POLICY_NAME',
        help='Name of the response policy.',
    )


def GetResponsePoliciesNameArg(positional=True, required=True):
  if positional:
    return base.Argument(
        'response_policies',
        type=str,
        metavar='RESPONSE_POLICY_NAME',
        help='Name of the response policy.',
    )
  else:
    return base.Argument(
        '--response_policies',
        type=str,
        required=required,
        metavar='RESPONSE_POLICY_NAME',
        help='Name of the response policy.',
    )


def GetLocationArg():
  return base.Argument(
      '--location',
      type=str,
      help=(
          'Specifies the desired service location the request is sent to.'
          ' Defaults to Cloud DNS global service. Use --location=global if you'
          ' want to target the global service.'
      ),
  )


def GetHealthCheckArg(required=False):
  return base.Argument(
      '--health-check',
      type=str,
      required=required,
      help=(
          'Specifies the health check to be used for public IP health checking.'
          ' Either the health check name or full resource path should be'
          ' provided.'
      ),
  )


def GetRoutingPolicyItemArg(required: bool = False) -> base.Argument:
  return base.Argument(
      '--routing-policy-item',
      type=arg_parsers.ArgDict(
          spec={
              'weight': float,
              'location': str,
              'rrdatas': str,
              'external_endpoints': str,
              'internal_load_balancers': str,
          }
      ),
      required=required,
      metavar='ROUTING_POLICY_ITEM',
      action='append',
      help="""\
    Specify this argument multiple times for a weighted round robin (WRR)
    or geolocation routing policy. Use this repeated argument for only one
    routing policy type (WRR or geolocation), not both. Similarly, use it
    only for health checking either internal load balancers or external IP
    addresses, not both.

    (e.g. --routing-policy-item=weight=1,rrdatas=1.2.3.4;2.3.4.5,external_endpoints=3.4.5.6;4.5.6.7
    --routing-policy-item=weight=1,rrdatas=10.0.0.1;10.0.0.2,external_endpoints=10.0.0.4)

    *weight*::: The weight of the item. This is specified only for WRR routing policy items.

    *location*::: The location corresponding to the item. This is specified only for GEO routing policy items.

    *rrdatas*::: The list of rrdatas, split by ";".

    *external_endpoints*::: The list of health checked ips, split by ";".

    *internal_load_balancers*::: The list of health checked internal load balancers, split by ";".
    """,
  )


def GetRoutingPolicyBackupItemArg(required: bool = False) -> base.Argument:
  return base.Argument(
      '--routing-policy-backup-item',
      type=arg_parsers.ArgDict(
          spec={
              'location': str,
              'rrdatas': str,
              'external_endpoints': str,
              'internal_load_balancers': str,
          }
      ),
      required=required,
      metavar='ROUTING_POLICY_BACKUP_ITEM',
      action='append',
      help="""\
    Specify this argument multiple times to define multiple items
    for a primary backup routing policy.

    (e.g. --routing-policy-backup-item=location=us-east1-a,rrdatas=1.2.3.4;2.3.4.5,external_endpoints=3.4.5.6;4.5.6.7
    --routing-policy-backup-item=location=us-east1-b,rrdatas=10.0.0.1;10.0.0.2,external_endpoints=10.0.0.4)

    *location*::: The location corresponding to the item.

    *rrdatas*::: The list of rrdatas, split by ";".

    *external_endpoints*::: The list of health checked ips, split by ";".

    *internal_load_balancers*::: The list of health checked internal load balancers, split by ";".
    """,
  )



# -*- coding: utf-8 -*- #
# Copyright 2025 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Shared flags definitions for finding commands."""



def AddParentOrFlagsGroup(parser):
  """Adds a mutually exclusive group that accepts either positional parent or --organization + --location."""
  group = parser.add_mutually_exclusive_group(required=True)

  # Positional parent argument
  group.add_argument(
      "PARENT",
      help=(
          "Parent of Cloud Security Command Center posture templates."
          " Formatted as organizations/<organizationID>/locations/<location>."
      ),
      nargs="?",
  )

  # Flag-based subgroup
  flags_group = group.add_argument_group(
      help="Specify organization and location using flags.",
  )
  flags_group.add_argument(
      "--organization",
      help="The organization ID (e.g., 123) that contains the resource.",
      required=True,
  )
  flags_group.add_argument(
      "--location",
      help=(
          "When data residency controls are enabled, this attribute specifies"
          " the location in which the resource is located and applicable."
      ),
      required=True,
  )
  return parser



# -*- coding: utf-8 -*- #
# Copyright 2024 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Interconnect Attachment Group."""




class InterconnectAttachmentGroup(object):
  """Abstracts Interconnect Attachment Group resource."""

  def __init__(self, ref, project, compute_client=None, resources=None):
    self.ref = ref
    self.project = project
    self._compute_client = compute_client
    self._resources = resources

  @property
  def _client(self):
    return self._compute_client.apitools_client

  @property
  def _messages(self):
    return self._compute_client.messages

  def _MakeAdditionalProperties(self, attachments):
    return [
        self._messages.InterconnectAttachmentGroup.AttachmentsValue.AdditionalProperty(
            # The keys are arbitrary strings that only need to
            # be unique, and we use region/attachment name.
            key=f'{region}/{attachment}',
            value=self._messages.InterconnectAttachmentGroupAttachment(
                attachment=self._resources.Create(
                    'compute.interconnectAttachments',
                    interconnectAttachment=attachment,
                    project=self.ref.project,
                    region=region,
                ).SelfLink()
            ),
        )
        for region, attachment in attachments
    ]

  def _MakeCreateRequestTuple(
      self,
      description,
      availability_sla,
      attachments,
  ):
    """Make a tuple for interconnect attachment group insert request.

    Args:
      description: String that represents the description of the Cloud
        Interconnect Attachment Group resource.
      availability_sla: String that represents the availability SLA of the Cloud
        Interconnect Attachment Group resource.
      attachments: List of strings that represent the names of the Cloud
        Interconnect Attachment resources that are members of the Cloud
        Interconnect Attachment Group resource.

    Returns:
    Insert interconnect attachment group tuple that can be used in a request.
    """
    messages = self._messages
    return (
        self._client.interconnectAttachmentGroups,
        'Insert',
        messages.ComputeInterconnectAttachmentGroupsInsertRequest(
            project=self.project,
            interconnectAttachmentGroup=messages.InterconnectAttachmentGroup(
                intent=messages.InterconnectAttachmentGroupIntent(
                    availabilitySla=availability_sla
                ),
                name=self.ref.Name(),
                description=description,
                attachments=messages.InterconnectAttachmentGroup.AttachmentsValue(
                    additionalProperties=self._MakeAdditionalProperties(
                        attachments
                    )
                ),
            ),
        ),
    )

  def _MakePatchRequestTuple(
      self, description, availability_sla, attachments, update_mask
  ):
    """Make a tuple for interconnect attachment group patch request."""
    messages = self._messages
    group_params = {
        'attachments': messages.InterconnectAttachmentGroup.AttachmentsValue(
            additionalProperties=self._MakeAdditionalProperties(attachments)
        ),
    }
    if description is not None:
      group_params['description'] = description
    if availability_sla is not None:
      group_params['intent'] = messages.InterconnectAttachmentGroupIntent(
          availabilitySla=availability_sla
      )
    return (
        self._client.interconnectAttachmentGroups,
        'Patch',
        messages.ComputeInterconnectAttachmentGroupsPatchRequest(
            project=self.project,
            interconnectAttachmentGroup=self.ref.Name(),
            interconnectAttachmentGroupResource=messages.InterconnectAttachmentGroup(
                **group_params
            ),
            updateMask=update_mask,
        ),
    )

  def _MakeDeleteRequestTuple(self):
    return (
        self._client.interconnectAttachmentGroups,
        'Delete',
        self._messages.ComputeInterconnectAttachmentGroupsDeleteRequest(
            project=self.ref.project,
            interconnectAttachmentGroup=self.ref.Name(),
        ),
    )

  def _MakeDescribeRequestTuple(self):
    return (
        self._client.interconnectAttachmentGroups,
        'Get',
        self._messages.ComputeInterconnectAttachmentGroupsGetRequest(
            project=self.ref.project,
            interconnectAttachmentGroup=self.ref.Name(),
        ),
    )

  def _MakeGetOperationalStatusRequestTuple(self):
    return (
        self._client.interconnectAttachmentGroups,
        'GetOperationalStatus',
        self._messages.ComputeInterconnectAttachmentGroupsGetOperationalStatusRequest(
            project=self.ref.project,
            interconnectAttachmentGroup=self.ref.Name(),
        ),
    )

  def Create(
      self,
      description=None,
      availability_sla=None,
      attachments=(),
      only_generate_request=False,
  ):
    """Create an interconnect attachment group."""
    requests = [
        self._MakeCreateRequestTuple(
            description,
            availability_sla,
            attachments,
        )
    ]
    if not only_generate_request:
      resources = self._compute_client.MakeRequests(requests)
      return resources[0]
    return requests

  def Patch(
      self,
      description=None,
      availability_sla=None,
      attachments=(),
      update_mask='',
      only_generate_request=False,
  ):
    """Patch an interconnect attachment group."""
    requests = [
        self._MakePatchRequestTuple(
            description, availability_sla, attachments, update_mask
        )
    ]
    if not only_generate_request:
      resources = self._compute_client.MakeRequests(requests)
      return resources[0]
    return requests

  def Delete(self, only_generate_request=False):
    requests = [self._MakeDeleteRequestTuple()]
    if not only_generate_request:
      return self._compute_client.MakeRequests(requests)
    return requests

  def Describe(self, only_generate_request=False):
    requests = [self._MakeDescribeRequestTuple()]
    if not only_generate_request:
      resources = self._compute_client.MakeRequests(requests)
      return resources[0]
    return requests

  def GetOperationalStatus(self, only_generate_request=False):
    requests = [self._MakeGetOperationalStatusRequestTuple()]
    if not only_generate_request:
      resources = self._compute_client.MakeRequests(requests)
      return resources[0]
    return requests



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utilities for calling the Composer ListWorkloads API."""


from googlecloudsdk.api_lib.composer import util as api_util
from googlecloudsdk.calliope import base

LIST_FIELD_NAME = 'workloads'
PAGE_SIZE = 1000


def GetService(release_track=base.ReleaseTrack.GA):
  return api_util.GetClientInstance(
      release_track).projects_locations_environments


class EnvironmentsWorkloadsService(object):
  """Provides workloads from Composer ListWorkloads API."""

  def __init__(self, release_track=base.ReleaseTrack.GA):
    self.client = None
    self.release_track = release_track
    self.messages = api_util.GetMessagesModule(release_track=self.release_track)

  def GetClient(self):
    if self.client is None:
      self.client = api_util.GetClientInstance(
          self.release_track).projects_locations_environments_workloads

    return self.client

  def List(self, project_location_ref):
    """Retrieves list of Composer workloads from Composer ListWorkloads API."""

    request = (
        self.messages.ComposerProjectsLocationsEnvironmentsWorkloadsListRequest
    )
    locations = [project_location_ref]

    return list(
        api_util.AggregateListResults(
            request, self.GetClient(), locations, LIST_FIELD_NAME, PAGE_SIZE
        )
    )



# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""This file provides the implementation of the `functions delete` command."""


from googlecloudsdk.api_lib.functions.v2 import exceptions
from googlecloudsdk.api_lib.functions.v2 import util as api_util
from googlecloudsdk.core import log
from googlecloudsdk.core.console import console_io


def Run(args, release_track):
  """Delete a Google Cloud Function."""
  client = api_util.GetClientInstance(release_track=release_track)
  messages = api_util.GetMessagesModule(release_track=release_track)

  function_ref = args.CONCEPTS.name.Parse()
  function_relative_name = function_ref.RelativeName()

  prompt_message = '2nd gen function [{0}] will be deleted.'.format(
      function_relative_name
  )
  if not console_io.PromptContinue(message=prompt_message):
    raise exceptions.FunctionsError('Deletion aborted by user.')

  operation = client.projects_locations_functions.Delete(
      messages.CloudfunctionsProjectsLocationsFunctionsDeleteRequest(
          name=function_relative_name))
  api_util.WaitForOperation(client, messages, operation, 'Deleting function')

  log.DeletedResource(function_relative_name)



# -*- coding: utf-8 -*- #
# Copyright 2014 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for creating HTTPS health checks."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute import completers
from googlecloudsdk.command_lib.compute.https_health_checks import flags


class CreateHttpsHealthCheck(base.CreateCommand):
  """Create a legacy HTTPS health check.

  Though you can use legacy HTTPS health checks in certain Google Cloud Platform
  load balancing configurations and for managed instance group autohealing, you
  should consider a non-legacy HTTPS health check created with `health-checks
  create https` instead.

  For more information about the differences between legacy and non-legacy
  health checks see:
  [](https://cloud.google.com/load-balancing/docs/health-check-concepts#category_and_protocol)

  For information about what type of health check to use for a particular load
  balancer, see:
  [](https://cloud.google.com/load-balancing/docs/health-check-concepts#lb_guide)
  """

  HTTPS_HEALTH_CHECKS_ARG = None

  @classmethod
  def Args(cls, parser):
    parser.display_info.AddFormat(flags.DEFAULT_LIST_FORMAT)
    cls.HTTPS_HEALTH_CHECKS_ARG = flags.HttpsHealthCheckArgument()
    cls.HTTPS_HEALTH_CHECKS_ARG.AddArgument(parser, operation_type='create')
    parser.display_info.AddCacheUpdater(completers.HttpsHealthChecksCompleter)

    parser.add_argument(
        '--host',
        help="""\
        The value of the host header used in this HTTPS health check request.
        By default, this is empty and Compute Engine automatically sets
        the host header in health requests to the same external IP address as
        the forwarding rule associated with the target pool.
        """)

    parser.add_argument(
        '--port',
        type=int,
        default=443,
        help="""\
        The TCP port number that this health check monitors. The default value
        is 443.
        """)

    parser.add_argument(
        '--request-path',
        default='/',
        help="""\
        The request path that this health check monitors. For example,
        ``/healthcheck''. The default value is ``/''.
        """)

    parser.add_argument(
        '--check-interval',
        type=arg_parsers.Duration(),
        default='5s',
        help="""\
        How often to perform a health check for an instance. For example,
        specifying ``10s'' will run the check every 10 seconds. The default
        value is ``5s''.
        See $ gcloud topic datetimes for information on duration formats.
        """)

    parser.add_argument(
        '--timeout',
        type=arg_parsers.Duration(),
        default='5s',
        help="""\
        If Compute Engine doesn't receive an HTTPS 200 response from the
        instance by the time specified by the value of this flag, the health
        check request is considered a failure. For example, specifying ``10s''
        will cause the check to wait for 10 seconds before considering the
        request a failure. The default value is ``5s''.
        See $ gcloud topic datetimes for information on duration formats.
        """)

    parser.add_argument(
        '--unhealthy-threshold',
        type=int,
        default=2,
        help="""\
        The number of consecutive health check failures before a healthy
        instance is marked as unhealthy. The default is 2.
        """)

    parser.add_argument(
        '--healthy-threshold',
        type=int,
        default=2,
        help="""\
        The number of consecutive successful health checks before an
        unhealthy instance is marked as healthy. The default is 2.
        """)

    parser.add_argument(
        '--description',
        help='An optional, textual description for the HTTPS health check.')

  def Run(self, args):
    """Issues the request necessary for adding the health check."""
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    client = holder.client

    health_check_ref = self.HTTPS_HEALTH_CHECKS_ARG.ResolveAsResource(
        args, holder.resources)

    request = client.messages.ComputeHttpsHealthChecksInsertRequest(
        httpsHealthCheck=client.messages.HttpsHealthCheck(
            name=health_check_ref.Name(),
            host=args.host,
            port=args.port,
            description=args.description,
            requestPath=args.request_path,
            checkIntervalSec=args.check_interval,
            timeoutSec=args.timeout,
            healthyThreshold=args.healthy_threshold,
            unhealthyThreshold=args.unhealthy_threshold,
        ),
        project=health_check_ref.project)

    return client.MakeRequests([(client.apitools_client.httpsHealthChecks,
                                 'Insert', request)])



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utilities for gkeonprem API clients for Bare Metal cluster resources."""


from typing import Optional

from apitools.base.py import exceptions as apitools_exceptions
from apitools.base.py import list_pager
from googlecloudsdk.api_lib.container.gkeonprem import client
from googlecloudsdk.api_lib.container.gkeonprem import update_mask
from googlecloudsdk.calliope import arg_parsers
from googlecloudsdk.calliope import exceptions
from googlecloudsdk.calliope import parser_extensions
from googlecloudsdk.core import properties
from googlecloudsdk.generated_clients.apis.gkeonprem.v1 import gkeonprem_v1_messages as messages


class _BareMetalClusterClient(client.ClientBase):
  """Base class for GKE OnPrem Bare Metal API clients."""

  def _create_annotations(self, args: parser_extensions.Namespace):
    """Constructs proto message AnnotationsValue for create command."""
    annotations = getattr(args, 'annotations', {})

    return self._dict_to_annotations_message(annotations)

  def _add_annotations(self, args: parser_extensions.Namespace):
    """Constructs proto message AnnotationsValue for adding new annotations."""
    result = {**self._get_current_annotations(args), **args.add_annotations}

    return self._dict_to_annotations_message(result)

  def _clear_annotations(self):
    """Constructs proto message AnnotationsValue for clearing annotations."""
    return messages.BareMetalCluster.AnnotationsValue()

  def _remove_annotations(self, args: parser_extensions.Namespace):
    """Constructs proto message AnnotationsValue for removing annotations."""
    curr_annotations = self._get_current_annotations(args)
    updated_annotations = {
        key: value
        for key, value in curr_annotations.items()
        if key not in args.remove_annotations
    }

    return self._dict_to_annotations_message(updated_annotations)

  def _set_annotations(self, args: parser_extensions.Namespace):
    """Constructs proto message AnnotationsValue for setting annotations."""
    return self._dict_to_annotations_message(args.set_annotations)

  def _get_current_annotations(self, args: parser_extensions.Namespace):
    """Fetches the standalone cluster annotations."""
    cluster_ref = args.CONCEPTS.cluster.Parse()
    cluster_response = self.Describe(cluster_ref)

    curr_annotations = {}
    if cluster_response.annotations:
      for annotation in cluster_response.annotations.additionalProperties:
        curr_annotations[annotation.key] = annotation.value

    return curr_annotations

  def _update_annotations(self, args: parser_extensions.Namespace):
    """Constructs proto message AnnotationsValue for update command."""
    specified_args = args.GetSpecifiedArgsDict()
    if 'add_annotations' in specified_args:
      return self._add_annotations(args)

    if 'clear_annotations' in specified_args:
      return self._clear_annotations()

    if 'remove_annotations' in specified_args:
      return self._remove_annotations(args)

    if 'set_annotations' in specified_args:
      return self._set_annotations(args)

  def _dict_to_annotations_message(self, annotations):
    """Converts key-val pairs to proto message AnnotationsValue."""
    additional_property_messages = []
    if not annotations:
      return None

    for key, value in annotations.items():
      additional_property_messages.append(
          messages.BareMetalCluster.AnnotationsValue.AdditionalProperty(
              key=key, value=value
          )
      )

    annotation_value_message = messages.BareMetalCluster.AnnotationsValue(
        additionalProperties=additional_property_messages
    )
    return annotation_value_message

  def _annotations(self, args: parser_extensions.Namespace):
    """Constructs proto message AnnotationsValue."""
    if args.command_path[-1] == 'update':
      return self._update_annotations(args)

    if args.command_path[-1] == 'create':
      return self._create_annotations(args)

    return None

  def _island_mode_cidr_config(self, args: parser_extensions.Namespace):
    """Constructs proto message BareMetalIslandModeCidrConfig."""
    kwargs = {
        'serviceAddressCidrBlocks': getattr(
            args, 'island_mode_service_address_cidr_blocks', []
        ),
        'podAddressCidrBlocks': getattr(
            args, 'island_mode_pod_address_cidr_blocks', []
        ),
    }

    if any(kwargs.values()):
      return messages.BareMetalIslandModeCidrConfig(**kwargs)

    return None

  def _sr_iov_config(self, args: parser_extensions.Namespace):
    kwargs = {
        'enabled': self.sr_iov_config_enabled(args),
    }

    if self.IsSet(kwargs):
      return messages.BareMetalSrIovConfig(**kwargs)

    return None

  def sr_iov_config_enabled(self, args: parser_extensions.Namespace):
    if 'enable_sr_iov_config' in args.GetSpecifiedArgsDict():
      return True
    elif 'disable_sr_iov_config' in args.GetSpecifiedArgsDict():
      return False
    else:
      return None

  def _multiple_network_interfaces_config(
      self, args: parser_extensions.Namespace
  ):
    kwargs = {
        'enabled': self.GetFlag(args, 'enable_multi_nic_config'),
    }
    if self.IsSet(kwargs):
      return messages.BareMetalMultipleNetworkInterfacesConfig(**kwargs)
    return None

  def _network_config(self, args: parser_extensions.Namespace):
    """Constructs proto message BareMetalNetworkConfig."""
    kwargs = {
        'islandModeCidr': self._island_mode_cidr_config(args),
        'advancedNetworking': self.GetFlag(args, 'enable_advanced_networking'),
        'multipleNetworkInterfacesConfig': (
            self._multiple_network_interfaces_config(args)
        ),
        'srIovConfig': self._sr_iov_config(args),
    }

    if self.IsSet(kwargs):
      return messages.BareMetalNetworkConfig(**kwargs)

    return None

  def _vip_config(self, args: parser_extensions.Namespace):
    """Constructs proto message BareMetalVipConfig."""
    kwargs = {
        'controlPlaneVip': getattr(args, 'control_plane_vip', None),
        'ingressVip': getattr(args, 'ingress_vip', None),
    }

    if any(kwargs.values()):
      return messages.BareMetalVipConfig(**kwargs)

    return None

  def _port_config(self, args: parser_extensions.Namespace):
    """Constructs proto message BareMetalPortConfig."""
    kwargs = {
        'controlPlaneLoadBalancerPort': getattr(
            args, 'control_plane_load_balancer_port', None
        ),
    }

    if any(kwargs.values()):
      return messages.BareMetalPortConfig(**kwargs)

    return None

  def _address_pools_from_file(self, args: parser_extensions.Namespace):
    """Constructs proto message field address_pools."""
    if not args.metal_lb_address_pools_from_file:
      return []

    address_pools = args.metal_lb_address_pools_from_file.get(
        'addressPools', []
    )

    if not address_pools:
      raise exceptions.BadArgumentException(
          '--metal_lb_address_pools_from_file',
          'Missing field [addressPools] in Metal LB address pools file.',
      )

    address_pool_messages = []
    for address_pool in address_pools:
      address_pool_messages.append(self._address_pool(address_pool))

    return address_pool_messages

  def _address_pool(self, address_pool):
    """Constructs proto message BareMetalLoadBalancerAddressPool."""
    addresses = address_pool.get('addresses', [])
    if not addresses:
      raise exceptions.BadArgumentException(
          '--metal_lb_address_pools_from_file',
          'Missing field [addresses] in Metal LB address pools file.',
      )

    pool = address_pool.get('pool', None)
    if not pool:
      raise exceptions.BadArgumentException(
          '--metal_lb_address_pools_from_file',
          'Missing field [pool] in Metal LB address pools file.',
      )

    kwargs = {
        'addresses': addresses,
        'avoidBuggyIps': address_pool.get('avoidBuggyIPs', None),
        'manualAssign': address_pool.get('manualAssign', None),
        'pool': pool,
    }

    return messages.BareMetalLoadBalancerAddressPool(**kwargs)

  def _address_pools_from_flag(self, args: parser_extensions.Namespace):
    if not args.metal_lb_address_pools:
      return []

    address_pools = []
    for address_pool in args.metal_lb_address_pools:
      address_pools.append(
          messages.BareMetalLoadBalancerAddressPool(
              addresses=address_pool.get('addresses', []),
              avoidBuggyIps=address_pool.get('avoid-buggy-ips', None),
              manualAssign=address_pool.get('manual-assign', None),
              pool=address_pool.get('pool', None),
          )
      )

    return address_pools

  def _metal_lb_node_config(self, metal_lb_node_config):
    """Constructs proto message BareMetalNodeConfig."""
    node_ip = metal_lb_node_config.get('nodeIP', '')
    if not node_ip:
      raise exceptions.BadArgumentException(
          '--metal_lb_load_balancer_node_configs_from_file',
          'Missing field [nodeIP] in Metal LB Node configs file.',
      )

    kwargs = {
        'nodeIp': node_ip,
        'labels': self._node_labels(metal_lb_node_config.get('labels', {})),
    }

    return messages.BareMetalNodeConfig(**kwargs)

  def _metal_lb_node_configs_from_file(self, args: parser_extensions.Namespace):
    """Constructs proto message field node_configs."""
    if not args.metal_lb_load_balancer_node_configs_from_file:
      return []

    metal_lb_node_configs = (
        args.metal_lb_load_balancer_node_configs_from_file.get(
            'nodeConfigs', []
        )
    )

    if not metal_lb_node_configs:
      raise exceptions.BadArgumentException(
          '--metal_lb_load_balancer_node_configs_from_file',
          'Missing field [nodeConfigs] in Metal LB Node configs file.',
      )

    metal_lb_node_configs_messages = []
    for metal_lb_node_config in metal_lb_node_configs:
      metal_lb_node_configs_messages.append(
          self._metal_lb_node_config(metal_lb_node_config)
      )

    return metal_lb_node_configs_messages

  def parse_node_labels(self, node_labels):
    """Validates and parses a node label object.

    Args:
      node_labels: str of key-val pairs separated by ';' delimiter.

    Returns:
      If label is valid, returns a dict mapping message LabelsValue to its
      value, otherwise, raise ArgumentTypeError.
      For example,
      {
          'key': LABEL_KEY
          'value': LABEL_VALUE
      }
    """
    if not node_labels.get('labels'):
      return None

    input_node_labels = node_labels.get('labels', '').split(';')
    additional_property_messages = []

    for label in input_node_labels:
      key_val_pair = label.split('=')
      if len(key_val_pair) != 2:
        raise arg_parsers.ArgumentTypeError(
            'Node Label [{}] not in correct format, expect KEY=VALUE.'.format(
                input_node_labels
            )
        )
      additional_property_messages.append(
          messages.BareMetalNodeConfig.LabelsValue.AdditionalProperty(
              key=key_val_pair[0], value=key_val_pair[1]
          )
      )

    labels_value_message = messages.BareMetalNodeConfig.LabelsValue(
        additionalProperties=additional_property_messages
    )

    return labels_value_message

  def node_config(self, node_config_args):
    """Constructs proto message BareMetalNodeConfig."""
    kwargs = {
        'nodeIp': node_config_args.get('node-ip', ''),
        'labels': self.parse_node_labels(node_config_args),
    }

    if any(kwargs.values()):
      return messages.BareMetalNodeConfig(**kwargs)

    return None

  def _metal_lb_node_configs_from_flag(self, args: parser_extensions.Namespace):
    """Constructs proto message field node_configs."""
    node_config_flag_value = (
        getattr(args, 'metal_lb_load_balancer_node_configs', [])
        if args.metal_lb_load_balancer_node_configs
        else []
    )

    return [
        self.node_config(node_config) for node_config in node_config_flag_value
    ]

  def _metal_lb_node_taints(self, args: parser_extensions.Namespace):
    """Constructs proto message NodeTaint."""
    taint_messages = []
    node_taints = getattr(args, 'metal_lb_load_balancer_node_taints', {})
    if not node_taints:
      return []

    for node_taint in node_taints.items():
      taint_object = self._parse_node_taint(node_taint)
      taint_messages.append(messages.NodeTaint(**taint_object))

    return taint_messages

  def _metal_lb_labels(self, args: parser_extensions.Namespace):
    """Constructs proto message LabelsValue."""
    node_labels = getattr(args, 'metal_lb_load_balancer_node_labels', {})
    additional_property_messages = []

    if not node_labels:
      return None

    for key, value in node_labels.items():
      additional_property_messages.append(
          messages.BareMetalNodePoolConfig.LabelsValue.AdditionalProperty(
              key=key, value=value
          )
      )

    labels_value_message = messages.BareMetalNodePoolConfig.LabelsValue(
        additionalProperties=additional_property_messages
    )

    return labels_value_message

  def _metal_lb_load_balancer_node_pool_config(
      self, args: parser_extensions.Namespace
  ):
    """Constructs proto message BareMetalNodePoolConfig."""
    if (
        'metal_lb_load_balancer_node_configs_from_file'
        in args.GetSpecifiedArgsDict()
    ):
      metal_lb_node_configs = self._metal_lb_node_configs_from_file(args)
    else:
      metal_lb_node_configs = self._metal_lb_node_configs_from_flag(args)

    kwargs = {
        'nodeConfigs': metal_lb_node_configs,
        'labels': self._metal_lb_labels(args),
        'taints': self._metal_lb_node_taints(args),
        'kubeletConfig': self._metal_lb_kubelet_config(args),
    }

    if any(kwargs.values()):
      return messages.BareMetalNodePoolConfig(**kwargs)
    return None

  def _metal_lb_serialize_image_pulls_disabled(
      self, args: parser_extensions.Namespace
  ):
    if (
        'enable_metal_lb_load_balancer_serialize_image_pulls'
        in args.GetSpecifiedArgsDict()
    ):
      return False
    elif (
        'disable_metal_lb_load_balancer_serialize_image_pulls'
        in args.GetSpecifiedArgsDict()
    ):
      return True
    else:
      return None

  def _metal_lb_kubelet_config(self, args: parser_extensions.Namespace):
    kwargs = {
        'registryBurst': self.GetFlag(
            args, 'metal_lb_load_balancer_registry_burst'
        ),
        'registryPullQps': self.GetFlag(
            args, 'metal_lb_load_balancer_registry_pull_qps'
        ),
        'serializeImagePullsDisabled': (
            self._metal_lb_serialize_image_pulls_disabled(args)
        ),
    }
    if self.IsSet(kwargs):
      return messages.BareMetalKubeletConfig(**kwargs)
    return None

  def _metal_lb_node_pool_config(self, args: parser_extensions.Namespace):
    """Constructs proto message BareMetalLoadBalancerNodePoolConfig."""
    kwargs = {
        'nodePoolConfig': self._metal_lb_load_balancer_node_pool_config(args),
    }

    if any(kwargs.values()):
      return messages.BareMetalLoadBalancerNodePoolConfig(**kwargs)

    return None

  def _metal_lb_config(self, args: parser_extensions.Namespace):
    """Constructs proto message BareMetalMetalLbConfig."""
    if 'metal_lb_address_pools_from_file' in args.GetSpecifiedArgsDict():
      address_pools = self._address_pools_from_file(args)
    else:
      address_pools = self._address_pools_from_flag(args)
    kwargs = {
        'addressPools': address_pools,
        'loadBalancerNodePoolConfig': self._metal_lb_node_pool_config(args),
    }

    if any(kwargs.values()):
      return messages.BareMetalMetalLbConfig(**kwargs)

    return None

  def _manual_lb_config(self, args: parser_extensions.Namespace):
    """Constructs proto message BareMetalManualLbConfig."""
    kwargs = {
        'enabled': getattr(args, 'enable_manual_lb', False),
    }

    if any(kwargs.values()):
      return messages.BareMetalManualLbConfig(**kwargs)

    return None

  def _load_balancer_config(self, args: parser_extensions.Namespace):
    """Constructs proto message BareMetalLoadBalancerConfig."""
    kwargs = {
        'vipConfig': self._vip_config(args),
        'portConfig': self._port_config(args),
        'metalLbConfig': self._metal_lb_config(args),
        'manualLbConfig': self._manual_lb_config(args),
        'bgpLbConfig': self._bgp_lb_config(args),
    }

    if any(kwargs.values()):
      return messages.BareMetalLoadBalancerConfig(**kwargs)

    return None

  def _bgp_lb_config(self, args: parser_extensions.Namespace):
    """Constructs proto message BareMetalBgpLbConfig."""
    kwargs = {
        'addressPools': self._bgp_address_pools(args),
        'asn': self.GetFlag(args, 'bgp_asn'),
        'bgpPeerConfigs': self._bgp_peer_configs(args),
        'loadBalancerNodePoolConfig': self._bgp_load_balancer_node_pool_config(
            args
        ),
    }

    if any(kwargs.values()):
      return messages.BareMetalBgpLbConfig(**kwargs)
    return None

  def _bgp_address_pools(self, args: parser_extensions.Namespace):
    """Constructs repeated proto message BareMetalBgpLbConfig.BareMetalLoadBalancerAddressPool."""
    if 'bgp_address_pools' not in args.GetSpecifiedArgsDict():
      return []

    address_pools = []
    for address_pool in args.bgp_address_pools:
      address_pools.append(
          messages.BareMetalLoadBalancerAddressPool(
              addresses=address_pool.get('addresses', []),
              avoidBuggyIps=address_pool.get('avoid-buggy-ips', None),
              manualAssign=address_pool.get('manual-assign', None),
              pool=address_pool.get('pool', None),
          )
      )

    return address_pools

  def _bgp_peer_configs(self, args: parser_extensions.Namespace):
    """Constructs repeated proto message BareMetalBgpPeerConfig."""
    if 'bgp_peer_configs' not in args.GetSpecifiedArgsDict():
      return []

    ret = []
    for peer_config in self.GetFlag(args, 'bgp_peer_configs'):
      msg = messages.BareMetalBgpPeerConfig(
          asn=peer_config.get('asn', None),
          controlPlaneNodes=peer_config.get('control-plane-nodes', []),
          ipAddress=peer_config.get('ip', None),
      )
      ret.append(msg)

    return ret

  def _bgp_load_balancer_node_pool_config(
      self, args: parser_extensions.Namespace
  ):
    """Constructs proto message BareMetalBgpLbConfig.BareMetalLoadBalancerNodePoolConfig."""
    kwargs = {
        'nodePoolConfig': self._bgp_node_pool_config(args),
    }
    if any(kwargs.values()):
      return messages.BareMetalLoadBalancerNodePoolConfig(**kwargs)

    return None

  def _bgp_node_pool_config(self, args: parser_extensions.Namespace):
    """Constructs proto message BareMetalBgpLbConfig.BareMetalLoadBalancerNodePoolConfig.BareMetalNodePoolConfig."""
    kwargs = {
        'labels': self._bgp_load_balancer_node_labels(args),
        'nodeConfigs': self._bgp_load_balancer_node_configs(args),
        'taints': self._bgp_load_balancer_node_taints(args),
        'kubeletConfig': self._bgp_load_balancer_kubelet_config(args),
    }
    if any(kwargs.values()):
      return messages.BareMetalNodePoolConfig(**kwargs)
    return None

  def _bgp_serialize_image_pulls_disabled(
      self, args: parser_extensions.Namespace
  ):
    if (
        'disable_bgp_load_balancer_serialize_image_pulls'
        in args.GetSpecifiedArgsDict()
    ):
      return True
    elif (
        'enable_bgp_load_balancer_serialize_image_pulls'
        in args.GetSpecifiedArgsDict()
    ):
      return False
    else:
      return None

  def _bgp_load_balancer_kubelet_config(
      self, args: parser_extensions.Namespace
  ):
    """Constructs proto message BareMetalBgpLbConfig.BareMetalLoadBalancerNodePoolConfig.BareMetalNodePoolConfig.BareMetalKubeletConfig."""
    kwargs = {
        'registryBurst': self.GetFlag(args, 'bgp_load_balancer_registry_burst'),
        'registryPullQps': self.GetFlag(
            args, 'bgp_load_balancer_registry_pull_qps'
        ),
        'serializeImagePullsDisabled': self._bgp_serialize_image_pulls_disabled(
            args
        ),
    }
    if self.IsSet(kwargs):
      return messages.BareMetalKubeletConfig(**kwargs)
    return None

  def _bgp_load_balancer_node_configs(self, args: parser_extensions.Namespace):
    """Constructs repeated proto message BareMetalBgpLbConfig.BareMetalLoadBalancerNodePoolConfig.BareMetalNodePoolConfig.BareMetalNodeConfig."""
    if 'bgp_load_balancer_node_configs' not in args.GetSpecifiedArgsDict():
      return []

    node_configs = []
    for node_config in self.GetFlag(args, 'bgp_load_balancer_node_configs'):
      node_configs.append(self.node_config(node_config))

    return node_configs

  def _bgp_load_balancer_node_labels(self, args: parser_extensions.Namespace):
    """Constructs proto message BareMetalBgpLbConfig.BareMetalLoadBalancerNodePoolConfig.BareMetalNodePoolConfig.LabelsValue."""
    if 'bgp_load_balancer_node_labels' not in args.GetSpecifiedArgsDict():
      return None

    additional_property_messages = []
    for key, value in args.bgp_load_balancer_node_labels.items():
      additional_property_messages.append(
          messages.BareMetalNodePoolConfig.LabelsValue.AdditionalProperty(
              key=key, value=value
          )
      )

    labels_value_message = messages.BareMetalNodePoolConfig.LabelsValue(
        additionalProperties=additional_property_messages
    )

    return labels_value_message

  def _bgp_load_balancer_node_taints(self, args: parser_extensions.Namespace):
    """Constructs repeated proto message NodeTaint."""
    if 'bgp_load_balancer_node_taints' not in args.GetSpecifiedArgsDict():
      return []

    node_taints = self.GetFlag(args, 'bgp_load_balancer_node_taints', {})
    taint_messages = []
    for node_taint in node_taints.items():
      taint_object = self._parse_node_taint(node_taint)
      taint_messages.append(messages.NodeTaint(**taint_object))

    return taint_messages

  def _lvp_config(self, args: parser_extensions.Namespace):
    """Constructs proto message BareMetalLvpConfig."""
    kwargs = {
        'path': getattr(args, 'lvp_share_path', None),
        'storageClass': getattr(args, 'lvp_share_storage_class', None),
    }

    if any(kwargs.values()):
      return messages.BareMetalLvpConfig(**kwargs)

    return None

  def _lvp_share_config(self, args: parser_extensions.Namespace):
    """Constructs proto message BareMetalLvpShareConfig."""
    kwargs = {
        'lvpConfig': self._lvp_config(args),
        'sharedPathPvCount': getattr(args, 'lvp_share_path_pv_count', None),
    }

    if any(kwargs.values()):
      return messages.BareMetalLvpShareConfig(**kwargs)

    return None

  def _lvp_node_mounts_config(self, args: parser_extensions.Namespace):
    """Constructs proto message BareMetalLvpConfig."""
    kwargs = {
        'path': getattr(args, 'lvp_node_mounts_config_path', None),
        'storageClass': getattr(
            args, 'lvp_node_mounts_config_storage_class', None
        ),
    }

    if any(kwargs.values()):
      return messages.BareMetalLvpConfig(**kwargs)

    return None

  def _storage_config(self, args: parser_extensions.Namespace):
    """Constructs proto message BareMetalStorageConfig."""
    kwargs = {
        'lvpShareConfig': self._lvp_share_config(args),
        'lvpNodeMountsConfig': self._lvp_node_mounts_config(args),
    }

    if any(kwargs.values()):
      return messages.BareMetalStorageConfig(**kwargs)

    return None

  def _node_labels(self, labels):
    """Constructs proto message LabelsValue."""
    additional_property_messages = []
    if not labels:
      return None

    for key, value in labels.items():
      additional_property_messages.append(
          messages.BareMetalNodeConfig.LabelsValue.AdditionalProperty(
              key=key, value=value
          )
      )

    labels_value_message = messages.BareMetalNodeConfig.LabelsValue(
        additionalProperties=additional_property_messages
    )

    return labels_value_message

  def _control_plane_node_config(self, control_plane_node_config):
    """Constructs proto message BareMetalNodeConfig."""
    node_ip = control_plane_node_config.get('nodeIP', '')
    if not node_ip:
      raise exceptions.BadArgumentException(
          '--control_plane_node_configs_from_file',
          'Missing field [nodeIP] in Control Plane Node configs file.',
      )

    kwargs = {
        'nodeIp': node_ip,
        'labels': self._node_labels(
            control_plane_node_config.get('labels', {})
        ),
    }

    return messages.BareMetalNodeConfig(**kwargs)

  def _control_plane_node_configs_from_file(
      self, args: parser_extensions.Namespace
  ):
    """Constructs proto message field node_configs."""
    if not args.control_plane_node_configs_from_file:
      return []

    control_plane_node_configs = args.control_plane_node_configs_from_file.get(
        'nodeConfigs', []
    )

    if not control_plane_node_configs:
      raise exceptions.BadArgumentException(
          '--control_plane_node_configs_from_file',
          'Missing field [nodeConfigs] in Control Plane Node configs file.',
      )

    control_plane_node_configs_messages = []
    for control_plane_node_config in control_plane_node_configs:
      control_plane_node_configs_messages.append(
          self._control_plane_node_config(control_plane_node_config)
      )

    return control_plane_node_configs_messages

  def _control_plane_node_configs_from_flag(
      self, args: parser_extensions.Namespace
  ):
    """Constructs proto message field node_configs."""
    node_configs = []
    node_config_flag_value = getattr(args, 'control_plane_node_configs', None)
    if node_config_flag_value:
      for node_config in node_config_flag_value:
        node_configs.append(self.node_config(node_config))

    return node_configs

  def _control_plane_node_taints(self, args: parser_extensions.Namespace):
    """Constructs proto message NodeTaint."""
    taint_messages = []
    node_taints = getattr(args, 'control_plane_node_taints', {})
    if not node_taints:
      return []

    for node_taint in node_taints.items():
      taint_object = self._parse_node_taint(node_taint)
      taint_messages.append(messages.NodeTaint(**taint_object))

    return taint_messages

  def _control_plane_node_labels(self, args: parser_extensions.Namespace):
    """Constructs proto message LabelsValue."""
    node_labels = getattr(args, 'control_plane_node_labels', {})
    additional_property_messages = []
    if not node_labels:
      return None

    for key, value in node_labels.items():
      additional_property_messages.append(
          messages.BareMetalNodePoolConfig.LabelsValue.AdditionalProperty(
              key=key, value=value
          )
      )

    labels_value_message = messages.BareMetalNodePoolConfig.LabelsValue(
        additionalProperties=additional_property_messages
    )

    return labels_value_message

  def _node_pool_config(self, args: parser_extensions.Namespace):
    """Constructs proto message BareMetalNodePoolConfig."""
    if 'control_plane_node_configs_from_file' in args.GetSpecifiedArgsDict():
      node_configs = self._control_plane_node_configs_from_file(args)
    else:
      node_configs = self._control_plane_node_configs_from_flag(args)

    kwargs = {
        'nodeConfigs': node_configs,
        'labels': self._control_plane_node_labels(args),
        'taints': self._control_plane_node_taints(args),
        'kubeletConfig': self._control_plane_kubelet_config(args),
    }

    if any(kwargs.values()):
      return messages.BareMetalNodePoolConfig(**kwargs)

    return None

  def _control_plane_serialize_image_pulls_disabled(
      self, args: parser_extensions.Namespace
  ):
    if (
        'disable_control_plane_serialize_image_pulls'
        in args.GetSpecifiedArgsDict()
    ):
      return True
    elif (
        'enable_control_plane_serialize_image_pulls'
        in args.GetSpecifiedArgsDict()
    ):
      return False
    else:
      return None

  def _control_plane_kubelet_config(self, args: parser_extensions.Namespace):
    kwargs = {
        'registryPullQps': self.GetFlag(
            args, 'control_plane_registry_pull_qps'
        ),
        'registryBurst': self.GetFlag(args, 'control_plane_registry_burst'),
        'serializeImagePullsDisabled': (
            self._control_plane_serialize_image_pulls_disabled(args)
        ),
    }
    if self.IsSet(kwargs):
      return messages.BareMetalKubeletConfig(**kwargs)
    return None

  def _control_plane_node_pool_config(self, args: parser_extensions.Namespace):
    """Constructs proto message BareMetalControlPlaneNodePoolConfig."""
    kwargs = {
        'nodePoolConfig': self._node_pool_config(args),
    }

    if any(kwargs.values()):
      return messages.BareMetalControlPlaneNodePoolConfig(**kwargs)

    return None

  def _api_server_args(self, args: parser_extensions.Namespace):
    """Constructs proto message BareMetalApiServerArgument."""
    api_server_args = []
    api_server_args_flag_value = getattr(args, 'api_server_args', None)
    if api_server_args_flag_value:
      for key, val in api_server_args_flag_value.items():
        api_server_args.append(
            messages.BareMetalApiServerArgument(argument=key, value=val)
        )

    return api_server_args

  def _control_plane_config(self, args: parser_extensions.Namespace):
    """Constructs proto message BareMetalControlPlaneConfig."""
    kwargs = {
        'controlPlaneNodePoolConfig': self._control_plane_node_pool_config(
            args
        ),
        'apiServerArgs': self._api_server_args(args),
    }

    if any(kwargs.values()):
      return messages.BareMetalControlPlaneConfig(**kwargs)

    return None

  def _proxy_config(self, args: parser_extensions.Namespace):
    """Constructs proto message BareMetalProxyConfig."""
    kwargs = {
        'uri': getattr(args, 'uri', None),
        'noProxy': getattr(args, 'no_proxy', []),
    }

    if any(kwargs.values()):
      return messages.BareMetalProxyConfig(**kwargs)

    return None

  def _cluster_operations_config(self, args: parser_extensions.Namespace):
    """Constructs proto message BareMetalClusterOperationsConfig."""
    kwargs = {
        'enableApplicationLogs': getattr(args, 'enable_application_logs', None),
    }

    if any(kwargs.values()):
      return messages.BareMetalClusterOperationsConfig(**kwargs)

    return None

  def _maintenance_config(self, args: parser_extensions.Namespace):
    """Constructs proto message BareMetalMaintenanceConfig."""
    kwargs = {
        'maintenanceAddressCidrBlocks': getattr(
            args, 'maintenance_address_cidr_blocks', []
        ),
    }

    if any(kwargs.values()):
      return messages.BareMetalMaintenanceConfig(**kwargs)

    return None

  def _container_runtime(self, container_runtime):
    """Constructs proto message BareMetalWorkloadNodeConfig.ContainerRuntimeValueValuesEnum."""
    if container_runtime is None:
      return None

    container_runtime_enum = (
        messages.BareMetalWorkloadNodeConfig.ContainerRuntimeValueValuesEnum
    )
    container_runtime_mapping = {
        'ContainerRuntimeUnspecified': (
            container_runtime_enum.CONTAINER_RUNTIME_UNSPECIFIED
        ),
        'Conatinerd': container_runtime_enum.CONTAINERD,
    }

    return container_runtime_mapping[container_runtime]

  def _workload_node_config(self, args: parser_extensions.Namespace):
    """Constructs proto message BareMetalWorkloadNodeConfig."""
    container_runtime = getattr(args, 'container_runtime', None)
    kwargs = {
        'containerRuntime': self._container_runtime(container_runtime),
        'maxPodsPerNode': getattr(args, 'max_pods_per_node', None),
    }

    if any(kwargs.values()):
      return messages.BareMetalWorkloadNodeConfig(**kwargs)

    return None

  # TODO(b/257292798): Move to common directory
  def _cluster_users(self, args: parser_extensions.Namespace):
    """Constructs repeated proto message ClusterUser."""
    cluster_user_messages = []
    admin_users = getattr(args, 'admin_users', None)
    if admin_users:
      return [
          messages.ClusterUser(username=admin_user)
          for admin_user in admin_users
      ]

    # On update, skip setting default value.
    if args.command_path[-1] == 'update':
      return None

    # On create, client side default admin user to the current gcloud user.
    gcloud_config_core_account = properties.VALUES.core.account.Get()
    if gcloud_config_core_account:
      default_admin_user_message = messages.ClusterUser(
          username=gcloud_config_core_account
      )
      return cluster_user_messages.append(default_admin_user_message)

    return None

  def _authorization(self, args: parser_extensions.Namespace):
    """Constructs proto message Authorization."""
    kwargs = {
        'adminUsers': self._cluster_users(args),
    }

    if any(kwargs.values()):
      return messages.Authorization(**kwargs)

    return None

  def _security_config(self, args: parser_extensions.Namespace):
    """Constructs proto message BareMetalSecurityConfig."""
    kwargs = {
        'authorization': self._authorization(args),
    }

    if any(kwargs.values()):
      return messages.BareMetalSecurityConfig(**kwargs)

    return None

  def _node_access_config(self, args: parser_extensions.Namespace):
    """Constructs proto message BareMetalNodeAccessConfig."""
    kwargs = {
        'loginUser': getattr(args, 'login_user', 'root'),
    }

    if any(kwargs.values()):
      return messages.BareMetalNodeAccessConfig(**kwargs)

    return None

  def _upgrade_policy(
      self, args: parser_extensions.Namespace
  ) -> Optional[messages.BareMetalClusterUpgradePolicy]:
    """Constructs proto message BareMetalClusterUpgradePolicy."""
    kwargs = {
        'controlPlaneOnly': getattr(args, 'upgrade_control_plane', None),
    }

    if any(kwargs.values()):
      return messages.BareMetalClusterUpgradePolicy(**kwargs)

    return None

  def _bare_metal_user_cluster(self, args: parser_extensions.Namespace):
    """Constructs proto message Bare Metal Cluster."""
    kwargs = {
        'name': self._user_cluster_name(args),
        'adminClusterMembership': self._admin_cluster_membership_name(args),
        'description': getattr(args, 'description', None),
        'annotations': self._annotations(args),
        'bareMetalVersion': getattr(args, 'version', None),
        'networkConfig': self._network_config(args),
        'controlPlane': self._control_plane_config(args),
        'loadBalancer': self._load_balancer_config(args),
        'storage': self._storage_config(args),
        'proxy': self._proxy_config(args),
        'clusterOperations': self._cluster_operations_config(args),
        'maintenanceConfig': self._maintenance_config(args),
        'nodeConfig': self._workload_node_config(args),
        'securityConfig': self._security_config(args),
        'nodeAccessConfig': self._node_access_config(args),
        'binaryAuthorization': self._binary_authorization(args),
        'upgradePolicy': self._upgrade_policy(args),
    }

    if any(kwargs.values()):
      return messages.BareMetalCluster(**kwargs)

    return None


class ClustersClient(_BareMetalClusterClient):
  """Client for clusters in gkeonprem bare metal API."""

  def __init__(self, **kwargs):
    super(ClustersClient, self).__init__(**kwargs)
    self._service = self._client.projects_locations_bareMetalClusters

  def List(self, args: parser_extensions.Namespace):
    """Lists Clusters in the GKE On-Prem Bare Metal API."""
    # Workaround for P4SA: Call query version config first, ignore the result.
    # Context: b/296435390#comment2
    project = (
        args.project if args.project else properties.VALUES.core.project.Get()
    )
    # Hard code location to `us-west1`, because it cannot handle `--location=-`.
    parent = 'projects/{project}/locations/{location}'.format(
        project=project, location='us-west1'
    )
    dummy_request = messages.GkeonpremProjectsLocationsBareMetalClustersQueryVersionConfigRequest(
        parent=parent,
    )
    try:
      _ = self._service.QueryVersionConfig(dummy_request)
    except (
        apitools_exceptions.HttpUnauthorizedError,
        apitools_exceptions.HttpForbiddenError,
    ):
      # Ignore unauthorized or forbidden errors. This may happen during the
      # support case when googler inspect customer's project via Google Admin.
      pass

    # If location is not specified, and container_bare_metal/location is not set
    # list clusters of all locations within a project.
    if (
        'location' not in args.GetSpecifiedArgsDict()
        and not properties.VALUES.container_bare_metal.location.Get()
    ):
      args.location = '-'

    list_req = messages.GkeonpremProjectsLocationsBareMetalClustersListRequest(
        parent=self._location_name(args)
    )

    return list_pager.YieldFromList(
        self._service,
        list_req,
        field='bareMetalClusters',
        batch_size=getattr(args, 'page_size', 100),
        limit=getattr(args, 'limit', None),
        batch_size_attribute='pageSize',
    )

  def Describe(self, resource_ref):
    """Gets a GKE On-Prem Bare Metal API cluster resource."""
    req = messages.GkeonpremProjectsLocationsBareMetalClustersGetRequest(
        name=resource_ref.RelativeName()
    )

    return self._service.Get(req)

  def Enroll(self, args: parser_extensions.Namespace):
    """Enrolls a bare metal cluster to Anthos."""
    kwargs = {
        'adminClusterMembership': self._admin_cluster_membership_name(args),
        'bareMetalClusterId': self._user_cluster_id(args),
        'localName': getattr(args, 'local_name', None),
    }
    enroll_bare_metal_cluster_request = messages.EnrollBareMetalClusterRequest(
        **kwargs
    )
    req = messages.GkeonpremProjectsLocationsBareMetalClustersEnrollRequest(
        parent=self._user_cluster_parent(args),
        enrollBareMetalClusterRequest=enroll_bare_metal_cluster_request,
    )

    return self._service.Enroll(req)

  def QueryVersionConfig(self, args: parser_extensions.Namespace):
    """Query Anthos on bare metal version configuration."""
    kwargs = {
        'createConfig_adminClusterMembership': (
            self._admin_cluster_membership_name(args)
        ),
        'upgradeConfig_clusterName': self._user_cluster_name(args),
        'parent': self._location_ref(args).RelativeName(),
    }

    req = messages.GkeonpremProjectsLocationsBareMetalClustersQueryVersionConfigRequest(
        **kwargs
    )
    return self._service.QueryVersionConfig(req)

  def Unenroll(self, args: parser_extensions.Namespace):
    """Unenrolls an Anthos cluster on bare metal."""
    kwargs = {
        'name': self._user_cluster_name(args),
        'force': getattr(args, 'force', None),
        'allowMissing': getattr(args, 'allow_missing', None),
    }
    req = messages.GkeonpremProjectsLocationsBareMetalClustersUnenrollRequest(
        **kwargs
    )

    return self._service.Unenroll(req)

  def Delete(self, args: parser_extensions.Namespace):
    """Deletes an Anthos cluster on bare metal."""
    kwargs = {
        'name': self._user_cluster_name(args),
        'allowMissing': getattr(args, 'allow_missing', False),
        'validateOnly': getattr(args, 'validate_only', False),
        'force': getattr(args, 'force', False),
        'ignoreErrors': self.GetFlag(args, 'ignore_errors'),
    }
    req = messages.GkeonpremProjectsLocationsBareMetalClustersDeleteRequest(
        **kwargs
    )

    return self._service.Delete(req)

  def Create(self, args: parser_extensions.Namespace):
    """Creates an Anthos cluster on bare metal."""
    kwargs = {
        'parent': self._user_cluster_parent(args),
        'validateOnly': getattr(args, 'validate_only', False),
        'bareMetalCluster': self._bare_metal_user_cluster(args),
        'bareMetalClusterId': self._user_cluster_id(args),
    }
    req = messages.GkeonpremProjectsLocationsBareMetalClustersCreateRequest(
        **kwargs
    )

    return self._service.Create(req)

  def Update(self, args: parser_extensions.Namespace):
    """Updates an Anthos cluster on bare metal."""
    kwargs = {
        'name': self._user_cluster_name(args),
        'allowMissing': self.GetFlag(args, 'allow_missing'),
        'updateMask': update_mask.get_update_mask(
            args, update_mask.BARE_METAL_CLUSTER_ARGS_TO_UPDATE_MASKS
        ),
        'validateOnly': self.GetFlag(args, 'validate_only'),
        'bareMetalCluster': self._bare_metal_user_cluster(args),
    }
    req = messages.GkeonpremProjectsLocationsBareMetalClustersPatchRequest(
        **kwargs
    )

    return self._service.Patch(req)

  def CreateFromImport(
      self,
      args: parser_extensions.Namespace,
      bare_metal_cluster,
      bare_metal_cluster_ref,
  ):
    """Creates an Anthos cluster on bare metal."""
    kwargs = {
        'parent': bare_metal_cluster_ref.Parent().RelativeName(),
        'validateOnly': self.GetFlag(args, 'validate_only'),
        'bareMetalCluster': bare_metal_cluster,
        'bareMetalClusterId': bare_metal_cluster_ref.Name(),
    }
    req = messages.GkeonpremProjectsLocationsBareMetalClustersCreateRequest(
        **kwargs
    )
    return self._service.Create(req)

  def UpdateFromFile(
      self, args: parser_extensions.Namespace, bare_metal_cluster
  ):
    # explicitly list supported mutable fields
    # etag is excluded from the mutable fields, because it is derived.
    top_level_mutable_fields = [
        'description',
        'bare_metal_version',
        'annotations',
        'network_config',
        'control_plane',
        'load_balancer',
        'storage',
        'proxy',
        'cluster_operations',
        'maintenance_config',
        'node_config',
        'security_config',
        'node_access_config',
        'os_environment_config',
    ]
    kwargs = {
        'name': self._user_cluster_name(args),
        'allowMissing': self.GetFlag(args, 'allow_missing'),
        'updateMask': ','.join(top_level_mutable_fields),
        'validateOnly': self.GetFlag(args, 'validate_only'),
        'bareMetalCluster': bare_metal_cluster,
    }
    req = messages.GkeonpremProjectsLocationsBareMetalClustersPatchRequest(
        **kwargs
    )
    return self._service.Patch(req)



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Defines arguments for BigLake commands."""

from googlecloudsdk.calliope import arg_parsers

# TODO(b/461544141): Move methods that define commands arguments from util.py
# to this file.


def AddDescriptionArg(parser):
  """Adds argument for description."""
  parser.add_argument(
      '--description',
      help='Description of the resource being created.',
  )


def AddPropertiesArg(parser):
  """Adds argument for creating properties."""
  parser.add_argument(
      '--properties',
      metavar='KEY=VALUE',
      type=arg_parsers.ArgDict(),
      help=(
          'Properties associated with the namespace.'
      ),
  )


def AddUpdatePropertiesArgs(parser):
  """Adds arguments for updating properties."""
  parser.add_argument(
      '--update-properties',
      metavar='KEY=VALUE',
      type=arg_parsers.ArgDict(),
      help=(
          'List of properties to update or add.'
      ),
  )
  parser.add_argument(
      '--remove-properties',
      metavar='KEY',
      type=arg_parsers.ArgList(),
      help=(
          'List of properties to remove.'
      ),
  )
  parser.add_argument(
      '--clear-properties',
      action='store_true',
      default=False,
      help=(
          'Clear all properties from the namespace.'
      ),
  )


def AddTableDescribeArgs(parser):
  """Adds arguments for describing tables."""
  parser.add_argument(
      '--snapshots',
      help='Snapshot to get.',
  )


def AddTableRegisterArgs(parser):
  """Adds arguments for registering tables."""
  parser.add_argument(
      '--metadata-location',
      required=True,
      help='Metadata location of the table.',
  )
  parser.add_argument(
      '--overwrite',
      action='store_true',
      default=False,
      help='Overwrite the table if it already exists.',
  )


def AddCatalogsCreateArgs(parser):
  """Adds arguments for creating catalogs."""
  parser.add_argument(
      '--primary-location',
      help=(
          'Primary location for mirroring the remote catalog metadata. It'
          ' must be a BigLake-supported location, and it should be proximate to'
          ' the remote catalog\'s location for better performance and lower'
          ' cost.'
      ),
  )


def AddServiceDirectoryNameArg(parser):
  """Adds argument for service directory name."""
  parser.add_argument(
      '--service-directory-name',
      hidden=True,
      help=(
          'The service directory resource name for request routing. See'
          ' cross-cloud interconnect documentation for more details.'
      ),
  )


def AddTableCreateArgs(parser):
  """Adds arguments for creating tables."""
  parser.add_argument(
      '--create-from-file',
      required=True,
      help=(
          'Path to a JSON file containing the table creation request. The'
          ' format must follow the Apache Iceberg REST Catalog Open API'
          ' specification for `CreateTableRequest`. The fields `name` and'
          ' `schema` must be specified.'
      ),
  )


def AddFederatedCatalogArgs(parser):
  """Adds arguments for federated catalogs."""
  parser.add_argument(
      '--federated-catalog-type',
      hidden=True,
      choices=['unity'],
      help='Type of the federated catalog.',
  )
  parser.add_argument(
      '--secret-name',
      hidden=True,
      help=(
          'The secret resource name in Secret Manager, in the format'
          ' `projects/{project_id}/locations/{location}/secrets/{secret_id}`'
          ' or `projects/{project_id}/locations/{location}/secrets/{secret_id}/'
          'versions/{version_id}`.'
      ),
  )
  parser.add_argument(
      '--unity-instance-name',
      hidden=True,
      help=(
          'The instance name is the first part of the URL when you log into'
          ' your Databricks deployment. For example, for a Databricks on GCP'
          ' workspace URL https://1.1.gcp.databricks.com, the instance name is'
          ' 1.1.gcp.databricks.com.'
      ),
  )
  parser.add_argument(
      '--unity-catalog-name',
      hidden=True,
      help='Name of the catalog in Unity Catalog.',
  )
  parser.add_argument(
      '--refresh-interval',
      hidden=True,
      type=arg_parsers.Duration(),
      help=(
          'The interval for refreshing metadata from the remote catalog (e.g.,'
          ' "1hr5m30s"). Defaults to seconds if no unit is specified.'
      ),
  )
  parser.add_argument(
      '--namespace-filters',
      hidden=True,
      type=arg_parsers.ArgList(),
      metavar='NAMESPACE',
      help=(
          'Filters to determine which namespaces are included in the refresh'
          ' process. - empty list means include all namespaces. -'
          ' "[namespaces]" means include the specified namespaces.'
      ),
  )


def AddUpdateFederatedCatalogArgs(parser):
  """Adds arguments for updating federated catalogs."""
  parser.add_argument(
      '--secret-name',
      hidden=True,
      help=(
          'Resource name of the Secret Manager secret, in the format'
          ' `projects/{project_id}/locations/{location}/secrets/{secret_id}`'
          ' or `projects/{project_id}/locations/{location}/secrets/{secret_id}/'
          'versions/{version_id}`.'
      ),
  )
  parser.add_argument(
      '--refresh-interval',
      hidden=True,
      type=arg_parsers.Duration(),
      help=(
          'Interval for refreshing metadata from the remote catalog (e.g.,'
          ' "1hr5m30s"). Defaults to seconds if no unit is specified.'
      ),
  )
  parser.add_argument(
      '--namespace-filters',
      hidden=True,
      type=arg_parsers.ArgList(),
      metavar='NAMESPACE',
      help=(
          'Filters to determine which namespaces are included in the refresh'
          ' process. - empty list means include all namespaces. -'
          ' "[namespaces]" means include the specified namespaces.'
      ),
  )



# -*- coding: utf-8 -*- #
# Copyright 2014 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command for listing snapshots."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.api_lib.compute import lister
from googlecloudsdk.calliope import base


def _GAArgs(parser):
  """Set Args for GA."""
  parser.display_info.AddFormat("""\
      table(
        name,
        diskSizeGb,
        sourceDisk.scope():label=SRC_DISK,
        status
      )""")
  lister.AddBaseListerArgs(parser)


def _BetaArgs(parser):
  """Set Args based on Release Track."""
  parser.display_info.AddFormat("""\
      table(
        name,
        location().yesno(no="GLOBAL"):label=LOCATION,
        diskSizeGb,
        sourceDisk.scope():label=SRC_DISK,
        status
      )""")
  lister.AddMultiScopeListerFlags(parser, global_=True, regional=True)


def _AlphaArgs(parser):
  """Set Args based on Release Track."""
  parser.display_info.AddFormat("""\
      table(
        name,
        location().yesno(no="GLOBAL"):label=LOCATION,
        diskSizeGb,
        sourceDisk.scope():label=SRC_DISK,
        status
      )""")
  lister.AddMultiScopeListerFlags(parser, global_=True, regional=True)


@base.ReleaseTracks(base.ReleaseTrack.GA, base.ReleaseTrack.PREVIEW)
@base.UniverseCompatible
class List(base.ListCommand):
  """List Compute Engine snapshots."""

  @staticmethod
  def Args(parser):
    _GAArgs(parser)

  def Run(self, args):
    return self._Run(args)

  def _Run(self, args, support_region=False):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    client = holder.client

    if support_region:
      request_data = lister.ParseMultiScopeFlags(args, holder.resources)

      list_implementation = lister.MultiScopeLister(
          client,
          global_service=client.apitools_client.snapshots,
          regional_service=client.apitools_client.regionSnapshots,
          aggregation_service=client.apitools_client.snapshots,
      )

      return lister.Invoke(request_data, list_implementation)
    else:
      request_data = lister.ParseNamesAndRegexpFlags(args, holder.resources)

      list_implementation = lister.GlobalLister(
          client, client.apitools_client.snapshots
      )

      return lister.Invoke(request_data, list_implementation)


List.detailed_help = base_classes.GetGlobalListerHelp('snapshots')


@base.ReleaseTracks(base.ReleaseTrack.BETA)
class ListBeta(List):
  """List Compute Engine snapshots."""

  @classmethod
  def Args(cls, parser):
    _BetaArgs(parser)

  def Run(self, args):
    return self._Run(args, support_region=True)


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class ListAlpha(List):
  """List Compute Engine snapshots."""

  @classmethod
  def Args(cls, parser):
    _AlphaArgs(parser)

  def Run(self, args):
    return self._Run(args, support_region=True)



# -*- coding: utf-8 -*- #
# Copyright 2020 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Delete github enterprise config command."""


from googlecloudsdk.api_lib.cloudbuild import cloudbuild_util
from googlecloudsdk.api_lib.util import waiter
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.builds import flags as build_flags
from googlecloudsdk.core import log
from googlecloudsdk.core import properties
from googlecloudsdk.core import resources


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class DeleteAlpha(base.DeleteCommand):
  """Delete a github enterprise config from Cloud Build."""

  @staticmethod
  def Args(parser):
    """Register flags for this command.

    Args:
      parser: An argparse.ArgumentParser-like object. It is mocked out in order
        to capture some information, but behaves like an ArgumentParser.
    """
    parser.add_argument('CONFIG', help='The id of the GitHub Enterprise Config')
    build_flags.AddRegionFlag(parser, hidden=False, required=False)

  def Run(self, args):
    """This is what gets called when the user runs this command.

    Args:
      args: an argparse namespace. All the arguments that were provided to this
        command invocation.

    Returns:
      Nothing on success.
    """

    client = cloudbuild_util.GetClientInstance()
    messages = cloudbuild_util.GetMessagesModule()

    parent = properties.VALUES.core.project.Get(required=True)
    regionprop = properties.VALUES.builds.region.Get()
    location = args.region or regionprop or cloudbuild_util.DEFAULT_REGION

    config_id = args.CONFIG

    # Get the github enterprise config ref
    ghe_resource = resources.REGISTRY.Parse(
        None,
        collection='cloudbuild.projects.locations.githubEnterpriseConfigs',
        api_version='v1',
        params={
            'projectsId': parent,
            'githubEnterpriseConfigsId': config_id,
            'locationsId': location,
        })

    # Send the Delete request
    deleted_op = client.projects_locations_githubEnterpriseConfigs.Delete(
        messages.CloudbuildProjectsLocationsGithubEnterpriseConfigsDeleteRequest(
            name=ghe_resource.RelativeName(),
            configId=config_id,
            projectId=parent))
    op_resource = resources.REGISTRY.ParseRelativeName(
        deleted_op.name, collection='cloudbuild.projects.locations.operations')
    waiter.WaitFor(
        waiter.CloudOperationPollerNoResources(
            client.projects_locations_operations), op_resource,
        'Deleting GitHub Enterprise Config')
    log.DeletedResource(ghe_resource)




# -*- coding: utf-8 -*- #
# Copyright 2021 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to list Tensorboards in Vertex AI."""


from googlecloudsdk.api_lib.ai.tensorboards import client
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.ai import constants
from googlecloudsdk.command_lib.ai import endpoint_util
from googlecloudsdk.command_lib.ai import flags
from googlecloudsdk.core import resources


def _GetUriGa(tensorboard):
  ref = resources.REGISTRY.ParseRelativeName(
      tensorboard.name,
      constants.TENSORBOARDS_COLLECTION,
      api_version=constants.AI_PLATFORM_API_VERSION[constants.GA_VERSION])
  return ref.SelfLink()


def _GetUriBeta(tensorboard):
  ref = resources.REGISTRY.ParseRelativeName(
      tensorboard.name,
      constants.TENSORBOARDS_COLLECTION,
      api_version=constants.AI_PLATFORM_API_VERSION[constants.BETA_VERSION])
  return ref.SelfLink()


def _Run(args, version):
  region_ref = args.CONCEPTS.region.Parse()
  region = region_ref.AsDict()['locationsId']
  with endpoint_util.AiplatformEndpointOverrides(
      version=version, region=region):
    return client.TensorboardsClient(version=version).List(
        limit=args.limit,
        page_size=args.page_size,
        region_ref=region_ref,
        sort_by=args.sort_by)


@base.ReleaseTracks(base.ReleaseTrack.GA)
class ListGa(base.ListCommand):
  """Lists the Tensorboards of the given project and region."""

  detailed_help = {
      'EXAMPLES':
          """\
          To list Tensorboards:

              $ {command}
          """,
  }

  @staticmethod
  def Args(parser):
    flags.AddRegionResourceArg(parser, 'to list Tensorboards')
    parser.display_info.AddUriFunc(_GetUriGa)

  def Run(self, args):
    return _Run(args, constants.GA_VERSION)


@base.ReleaseTracks(base.ReleaseTrack.BETA, base.ReleaseTrack.ALPHA)
class ListBeta(base.ListCommand):
  """Lists the Tensorboards of the given project and region."""

  detailed_help = {
      'EXAMPLES':
          """\
          To list Tensorboards:

              $ {command}
          """,
  }

  @staticmethod
  def Args(parser):
    flags.AddRegionResourceArg(parser, 'to list Tensorboards')
    parser.display_info.AddUriFunc(_GetUriBeta)

  def Run(self, args):
    return _Run(args, constants.BETA_VERSION)



# -*- coding: utf-8 -*- #
# Copyright 2023 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Command for listing bgp routes from a Compute Engine router."""


from apitools.base.py import list_pager
from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.compute import flags as compute_flags
from googlecloudsdk.command_lib.compute.routers import flags
from googlecloudsdk.command_lib.util.apis import arg_utils


@base.UniverseCompatible
class ListBgpRoutes(base.ListCommand):
  """List routes advertised and learned on individual BGP sessions, both pre- and post-policy evaluation."""

  ROUTER_ARG = None

  @classmethod
  def Args(cls, parser):
    ListBgpRoutes.ROUTER_ARG = flags.RouterArgument()
    ListBgpRoutes.ROUTER_ARG.AddArgument(parser, operation_type='list')
    parser.display_info.AddCacheUpdater(flags.RoutersCompleter)
    parser.add_argument(
        '--peer',
        help="""Limit results to routes learned from this peer (name).""",
        required=True,
    )
    parser.add_argument(
        '--address-family',
        type=arg_utils.ChoiceToEnumName,
        choices={
            'IPV4': 'Interface with IPv4-based BGP.',
            'IPV6': 'Interface with IPv6-based BGP.',
        },
        help="""Limit results to routes learned for this Address Family Identifier.""",
        required=True,
    )
    parser.add_argument(
        '--route-direction',
        type=arg_utils.ChoiceToEnumName,
        choices={
            'INBOUND': 'Learned routes.',
            'OUTBOUND': 'Advertised routes.',
        },
        help="""Limit results to routes in this direction.""",
        required=True,
    )
    parser.add_argument(
        '--policy-applied',
        action='store_true',
        default=True,
        help="""Routes returned are post-policy evaluation.""",
    )
    parser.add_argument(
        '--destination-range',
        help="""Limit results to prefixes.""",
        metavar='CIDR_RANGE',
    )

  def Run(self, args):
    """Issues a request necessary for listing bgp routes from a Router."""
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    client = holder.client

    router_ref = ListBgpRoutes.ROUTER_ARG.ResolveAsResource(
        args,
        holder.resources,
        scope_lister=compute_flags.GetDefaultScopeLister(client),
    )
    address_family = client.messages.ComputeRoutersListBgpRoutesRequest.AddressFamilyValueValuesEnum(
        args.address_family
    )
    route_type = client.messages.ComputeRoutersListBgpRoutesRequest.RouteTypeValueValuesEnum(
        self.ConvertRouteDirection(args.route_direction)
    )

    request = client.messages.ComputeRoutersListBgpRoutesRequest(
        **router_ref.AsDict(),
        peer=args.peer,
        addressFamily=address_family,
        routeType=route_type,
        policyApplied=args.policy_applied,
        destinationPrefix=args.destination_range,
    )
    return list_pager.YieldFromList(
        client.apitools_client.routers,
        request,
        limit=args.limit,
        batch_size=args.page_size,
        method='ListBgpRoutes',
        field='result',
        current_token_attribute='pageToken',
        next_token_attribute='nextPageToken',
        batch_size_attribute='maxResults',
    )

  def ConvertRouteDirection(self, route_direction):
    if route_direction == 'INBOUND':
      return 'LEARNED'
    elif route_direction == 'OUTBOUND':
      return 'ADVERTISED'
    else:
      return route_direction


ListBgpRoutes.detailed_help = {
    'DESCRIPTION': """\
  *{command}* lists routes advertised and learned on individual BGP sessions, both pre- and post-policy evaluation.
  """,
    'EXAMPLES': """\
          To list inbound BGP routes limited to IPv4 addess family from a router `my-router` BGP peer `my-bgp-peer` in region `us-central1`, run:

              $ {command} my-router --region=us-central1 --address-family=IPV4 --peer=my-bgp-peer --route-direction=INBOUND"
          """,
}



# -*- coding: utf-8 -*- #
# Copyright 2026 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Utilities for building Python environments for Orchestration Pipelines."""

import contextlib

from googlecloudsdk.calliope import exceptions
from googlecloudsdk.core import log
from googlecloudsdk.core.util import files


@contextlib.contextmanager
def _temp_build_dir(work_dir):
  """Context manager for a temporary build directory."""
  build_root = work_dir / 'temp_build_libs'
  try:
    files.RmTree(build_root)
  except FileNotFoundError:
    pass
  build_root.mkdir(parents=True, exist_ok=True)
  try:
    yield build_root
  finally:
    try:
      files.RmTree(build_root)
    except FileNotFoundError:
      pass


def build_env_local(
    subprocess_mod,
    work_dir,
    requirements_file_path,
    output_tar_gz_path,
    python_version,
):
  """Builds dependencies.tar.gz locally using uv and venv-pack.

  Args:
    subprocess_mod: The subprocess module or mock.
    work_dir: The working directory as a pathlib.Path object.
    requirements_file_path: Path to requirements.txt file.
    output_tar_gz_path: The path for the output tar.gz file.
    python_version: The target Python version for pip install.
  """

  with _temp_build_dir(work_dir) as build_root:
    venv_path = build_root / 'deployment_env'
    output_path = str(output_tar_gz_path)

    try:
      # 1. Create the virtual environment using uv
      log.info('Creating virtual environment with uv...')
      subprocess_mod.run(
          [
              'uv',
              'venv',
              str(venv_path),
              '--python',
              python_version,
              '--relocatable',
          ],
          check=True,
          capture_output=True,
          text=True,
      )

      # 2. Install requirements using uv pip
      log.info('Installing requirements using uv pip...')
      pip_install_cmd = [
          'uv',
          'pip',
          'install',
          '--python',
          str(venv_path),
          '-r',
          str(requirements_file_path),
          '--no-cache',
          # Copying is required to have all libs in site packages.
          '--link-mode=copy',
          # We host resulting spark jobs on Linux VMs.
          '--python-platform',
          'linux',
          # Do not build from source as arch can be different.
          '--only-binary=:all:',
      ]

      subprocess_mod.run(
          pip_install_cmd, check=True, capture_output=True, text=True
      )

      # 3. Pack the environment using tar
      log.info('Packing python environment...')
      subprocess_mod.run(
          [
              'tar',
              '-czf',
              output_path,
              '-C',
              str(venv_path),
              '.',
          ],
          check=True,
          capture_output=True,
          text=True,
      )

    except subprocess_mod.CalledProcessError as e:
      log.error('Command failed: %s', ' '.join(e.cmd))
      if e.stderr:
        log.error('Stderr:\n%s', e.stderr)
      raise exceptions.ToolException('Local build failed') from e



# -*- coding: utf-8 -*- #
# Copyright 2017 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to update the project."""


from googlecloudsdk.api_lib.compute import base_classes
from googlecloudsdk.calliope import base
from googlecloudsdk.core import properties


@base.DefaultUniverseOnly
@base.ReleaseTracks(base.ReleaseTrack.GA, base.ReleaseTrack.PREVIEW)
class Update(base.UpdateCommand):
  r"""Update a Compute Engine project resource.

  *{command}* is used to update a Compute Engine project resource.
  """

  _support_managed_protection_tier = False

  @classmethod
  def Args(cls, parser):
    parser.add_argument(
        '--default-network-tier',
        choices=['PREMIUM', 'STANDARD'],
        type=lambda x: x.upper(),
        help='The default network tier to assign to the project.')
    parser.add_argument(
        '--cloud-armor-tier',
        choices=['CA_STANDARD', 'CA_ENTERPRISE_PAYGO', 'CA_ENTERPRISE_ANNUAL'],
        type=lambda x: x.upper(),
        help='Cloud armor tier to assign to the project.',
    )
    if cls._support_managed_protection_tier:
      parser.add_argument(
          '--managed-protection-tier',
          choices=['CA_STANDARD', 'CAMP_PLUS_PAYGO', 'CAMP_PLUS_ANNUAL'],
          type=lambda x: x.upper(),
          help='Managed protection tier to assign to the project.',
      )

  def Run(self, args):
    holder = base_classes.ComputeApiHolder(self.ReleaseTrack())
    client = holder.client.apitools_client
    messages = holder.client.messages

    requests = []

    if args.default_network_tier:
      request = messages.ComputeProjectsSetDefaultNetworkTierRequest(
          project=properties.VALUES.core.project.GetOrFail(),
          projectsSetDefaultNetworkTierRequest=messages.
          ProjectsSetDefaultNetworkTierRequest(
              networkTier=messages.ProjectsSetDefaultNetworkTierRequest.
              NetworkTierValueValuesEnum(args.default_network_tier)))
      requests.append((client.projects, 'SetDefaultNetworkTier', request))
    if args.cloud_armor_tier:
      request = messages.ComputeProjectsSetCloudArmorTierRequest(
          project=properties.VALUES.core.project.GetOrFail(),
          projectsSetCloudArmorTierRequest=messages.ProjectsSetCloudArmorTierRequest(
              cloudArmorTier=messages.ProjectsSetCloudArmorTierRequest.CloudArmorTierValueValuesEnum(
                  args.cloud_armor_tier
              )
          ),
      )
      requests.append((client.projects, 'SetCloudArmorTier', request))
    elif self._support_managed_protection_tier and args.managed_protection_tier:
      request = messages.ComputeProjectsSetManagedProtectionTierRequest(
          project=properties.VALUES.core.project.GetOrFail(),
          projectsSetManagedProtectionTierRequest=messages.ProjectsSetManagedProtectionTierRequest(
              managedProtectionTier=messages.ProjectsSetManagedProtectionTierRequest.ManagedProtectionTierValueValuesEnum(
                  args.managed_protection_tier
              )
          ),
      )
      requests.append((client.projects, 'SetManagedProtectionTier', request))

    return holder.client.MakeRequests(requests)


@base.ReleaseTracks(base.ReleaseTrack.BETA)
class UpdateBeta(Update):
  r"""Update a Compute Engine project resource.

  *{command}* is used to update a Compute Engine project resource.
  """

  _support_managed_protection_tier = True


@base.ReleaseTracks(base.ReleaseTrack.ALPHA)
class UpdateAlpha(UpdateBeta):
  r"""Update a Compute Engine project resource.

  *{command}* is used to update a Compute Engine project resource.
  """

  _support_managed_protection_tier = True



# -*- coding: utf-8 -*- # Lint as: python3
# Copyright 2020 Google Inc. All Rights Reserved.
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Command to describe an Apigee API deployment."""


from googlecloudsdk.api_lib import apigee
from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.apigee import defaults
from googlecloudsdk.command_lib.apigee import errors
from googlecloudsdk.command_lib.apigee import resource_args


class Describe(base.DescribeCommand):
  """Describe an Apigee API proxy deployment."""

  detailed_help = {
      "DESCRIPTION":
          """\
  {description}

  `{command}` retrieves the status of a specific API proxy's deployment to a
  specific environment.
          """,
      "EXAMPLES":
          """\
  To get the status of a deployment in the active Cloud Platform project, which
  deploys ``my-proxy'' into the ``prod'' environment, run:

      $ {command} --api=my-proxy --environment=prod

  To get the status of that deployment as a JSON object, run:

      $ {command} --api=my-proxy --environment=prod --format=json

  To get the status of a deployment in an organization called ``my-org'', which
  deploys version 3 of the proxy ``my-proxy'' into the ``test'' environment,
  run:

      $ {command} 3 --organization=my-org --environment=test --api=my-proxy
          """
  }

  @staticmethod
  def Args(parser):
    help_text = {
        "api": "Deployed API proxy.",
        "environment": "Environment in which the proxy was deployed.",
        "organization": ("Apigee Organization of the proxy and environment. If "
                         "unspecified, the Cloud Platform project's associated "
                         "organization will be used."),
    }
    fallthroughs = [
        defaults.GCPProductOrganizationFallthrough(),
        defaults.StaticFallthrough("revision", "auto")
    ]
    resource_args.AddSingleResourceArgument(
        parser,
        "organization.environment.api.revision",
        """\
API proxy revision and environment of the deployment to be described.


To get a list of deployed proxies and their environments, run:

    $ {parent_command} list

REVISION can either be a positive revision number, or the special value
``auto'', which will choose whichever revision of API is currently deployed in
ENVIRONMENT, or return an error if more than one revision is deployed.


If REVISION is unspecified, the default is ``auto''.

""",
        fallthroughs=fallthroughs,
        help_texts=help_text)

  def Run(self, args):
    """Run the describe command."""
    identifiers = args.CONCEPTS.revision.Parse().AsDict()
    if identifiers["revisionsId"] == "auto":
      del identifiers["revisionsId"]
      defaults.FallBackToDeployedProxyRevision(identifiers)

    # Deployments have no identifier of their own, so the API to get deployment
    # details looks like a List call with all possible parents specified.
    deployments = apigee.DeploymentsClient.List(identifiers)
    if not deployments:
      raise errors.EntityNotFoundError("deployment", identifiers, "GET")
    return deployments[0]



# -*- coding: utf-8 -*- #
# Copyright 2022 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Command to rename named configuration."""


from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.config import completers
from googlecloudsdk.core import log
from googlecloudsdk.core.configurations import named_configs


class Rename(base.SilentCommand):
  """Renames a named configuration."""

  detailed_help = {
      'DESCRIPTION': """\
          {description}

          See `gcloud topic configurations` for an overview of named
          configurations.
          """,
      'EXAMPLES': """\
          To rename an existing configuration named `my-config`, run:

            $ {command} my-config --new-name=new-config
          """,
  }

  @staticmethod
  def Args(parser):
    """Adds args for this command."""
    parser.add_argument(
        'configuration_name',
        completer=completers.NamedConfigCompleter,
        help='Name of the configuration to rename')
    parser.add_argument(
        '--new-name',
        required=True,
        help='Specifies the new name of the configuration.')

  def Run(self, args):

    named_configs.ConfigurationStore.RenameConfig(args.configuration_name,
                                                  args.new_name)

    log.status.Print('Renamed [{0}] to be [{1}].'.format(
        args.configuration_name, args.new_name))

    return args.new_name



# -*- coding: utf-8 -*- #
# Copyright 2015 Google LLC. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

"""Command to activate named configuration."""


from googlecloudsdk.calliope import base
from googlecloudsdk.command_lib.config import completers
from googlecloudsdk.command_lib.config import config_validators
from googlecloudsdk.command_lib.projects import util as projects_util
from googlecloudsdk.core import log
from googlecloudsdk.core import properties
from googlecloudsdk.core.configurations import named_configs


@base.UniverseCompatible
class Activate(base.SilentCommand):
  """Activates an existing named configuration."""

  detailed_help = {
      'DESCRIPTION': """\
          {description}

          See `gcloud topic configurations` for an overview of named
          configurations.
          """,
      'EXAMPLES': """\
          To activate an existing configuration named `my-config`, run:

            $ {command} my-config

          To list all properties in the activated configuration, run:

            $ gcloud config list --all
          """,
  }

  @staticmethod
  def Args(parser):
    """Adds args for this command."""
    parser.add_argument(
        'configuration_name',
        completer=completers.NamedConfigCompleter,
        help='Name of the configuration to activate')

  def Run(self, args):
    named_configs.ConfigurationStore.ActivateConfig(args.configuration_name)
    log.status.write('Activated [{0}].\n'.format(args.configuration_name))
    project_id = properties.VALUES.core.project.Get()
    if project_id:
      # Warning if current project does not match the one in ADC
      config_validators.WarnIfSettingProjectWhenAdcExists(project_id)
      projects_util.PrintEnvironmentTagMessageProjectId(project_id)

    return args.configuration_name

